//-------------------------------------------------------------------------------------------------------
//
// DE_Auswertung
//
//-------------------------------------------------------------------------------------------------------

#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "Funktionen.h"
#include "Module.h"


//-------------------------------------------------------------------------------------------------------
// Initialisierung

void F_DE_init(void)
{
	// 24V Eingnge (DE1-14)
	DDRA = 0x00;
	PORTA = 0xFF;
	DDRB &= ~((1<<0) | (1<<1));
	PORTB |= (1<<0) | (1<<1);
	DDRC &= ~((1<<4) | (1<<5) | (1<<6) | (1<<7));
	PORTC |= (1<<4) | (1<<5) | (1<<6) | (1<<7);
}


//-------------------------------------------------------------------------------------------------------
// Entprellfunktionen

// Prft ob ein DE bettigt wurde. Jede Bettigung wird nur 1x erkannt.
uint16_t F_DE_erkannt(uint16_t DE_Maske)
{
	cli();
	DE_Maske &= DE_Erkannt;  // DEs lesen
	DE_Erkannt ^= DE_Maske;  // erkannte DEs lschen
	sei();
	return DE_Maske;
}

// Prft ob ein DE lang genug bettigt wurde damit die Wiederholfunktion startet.
// Diese Funktion simuliert, dass der DE toggelt.
uint16_t F_DE_repeat(uint16_t DE_Maske)
{
	cli();
	DE_Maske &= DE_Repeat;  // DEs lesen
	DE_Repeat ^= DE_Maske;  // erkannte DEs lschen
	sei();
	return DE_Maske;
}

// Prft ob ein DE genau jetzt bettigt ist
uint16_t F_DE_status(uint16_t DE_Maske)
{
	DE_Maske &= DE_Status;
	return DE_Maske;
}

// Prft ob ein DE 1x kurz bettigt wurde
uint16_t F_DE_1kurz(uint16_t DE_Maske)
{
	cli();
	return F_DE_erkannt(~DE_Status & DE_Maske & ~DE_Zwei);
}

// Prft ob ein DE 2x kurz bettigt wurde
uint16_t F_DE_2kurz(uint16_t DE_Maske)
{
	cli();
	return F_DE_erkannt(~DE_Status & DE_Maske & DE_Zwei);
}

// Prft ob ein DE lang bettigt wurde
uint16_t F_DE_lang(uint16_t DE_Maske)
{
	return F_DE_erkannt(F_DE_repeat(DE_Maske));
}

// Prft ob fallende Flanke
uint16_t F_DE_fla_fall(uint16_t DE_Maske)
{
	cli();
	DE_Maske &= DE_Flanke_fall;  // DEs lesen
	DE_Flanke_fall ^= DE_Maske;  // erkannte DEs lschen
	sei();
	return DE_Maske;
}

// Prft ob steigende Flanke
uint16_t F_DE_fla_steig(uint16_t DE_Maske)
{
	cli();
	DE_Maske &= DE_Flanke_steig;  // DEs lesen
	DE_Flanke_steig ^= DE_Maske;  // erkannte DEs lschen
	sei();
	return DE_Maske;
}
