//-------------------------------------------------------------------------------------------------------
//
// Dimmeransteuerung ber PWM
//
// verwendeter Dimmer: Eltako LUD12-230V
//	- PWM Frequenz = 100Hz
//	- Dimmer aus = 0% PWM
//	- Dimmer 100% ein = 90% PWM
//
//-------------------------------------------------------------------------------------------------------


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "Config.h"
#include "SPI.h"
#include "EEPROM.h"
#include "DE.h"


//-------------------------------------------------------------------------------------------------------
// PWM Dimmer initialisieren
void F_Dimmer_init(void)
{
	uint16_t Dimmer_Log[101];	
	uint8_t i=0;


	// Logarithmische Dimmkurve
	Dimmer_Log[0] = 0,		Dimmer_Log[17] = 31;		Dimmer_Log[34] = 100;
	Dimmer_Log[1] = 9;		Dimmer_Log[18] = 33;		Dimmer_Log[35] = 107;
	Dimmer_Log[2] = 10;		Dimmer_Log[19] = 35;		Dimmer_Log[36] = 115;
	Dimmer_Log[3] = 11;		Dimmer_Log[20] = 38;		Dimmer_Log[37] = 123;
	Dimmer_Log[4] = 12;		Dimmer_Log[21] = 40;		Dimmer_Log[38] = 132;
	Dimmer_Log[5] = 13;		Dimmer_Log[22] = 43;		Dimmer_Log[39] = 142;
	Dimmer_Log[6] = 14;		Dimmer_Log[23] = 46;		Dimmer_Log[40] = 152;
	Dimmer_Log[7] = 15;		Dimmer_Log[24] = 49;		Dimmer_Log[41] = 163;
	Dimmer_Log[8] = 16;		Dimmer_Log[25] = 53;		Dimmer_Log[42] = 175;
	Dimmer_Log[9] = 18;		Dimmer_Log[26] = 57;		Dimmer_Log[43] = 187;
	Dimmer_Log[10] = 19;		Dimmer_Log[27] = 61;		Dimmer_Log[44] = 201;
	Dimmer_Log[11] = 20;		Dimmer_Log[28] = 66;		Dimmer_Log[45] = 215;
	Dimmer_Log[12] = 21;		Dimmer_Log[29] = 71;		Dimmer_Log[46] = 231;
	Dimmer_Log[13] = 23;		Dimmer_Log[30] = 76;		Dimmer_Log[47] = 248;
	Dimmer_Log[14] = 25;		Dimmer_Log[31] = 81;		Dimmer_Log[48] = 266;
	Dimmer_Log[15] = 27;		Dimmer_Log[32] = 87;		Dimmer_Log[49] = 285;
	Dimmer_Log[16] = 29;		Dimmer_Log[33] = 93;		Dimmer_Log[50] = 305;
	
	Dimmer_Log[51] = 328;		Dimmer_Log[68] = 1072;		Dimmer_Log[85] = 3511;
	Dimmer_Log[52] = 351;		Dimmer_Log[69] = 1150;		Dimmer_Log[86] = 3765;
	Dimmer_Log[53] = 377;		Dimmer_Log[70] = 1233;		Dimmer_Log[87] = 4037;
	Dimmer_Log[54] = 404;		Dimmer_Log[71] = 1322;		Dimmer_Log[88] = 4329;
	Dimmer_Log[55] = 433;		Dimmer_Log[72] = 1418;		Dimmer_Log[89] = 4642;
	Dimmer_Log[56] = 464;		Dimmer_Log[73] = 1520;		Dimmer_Log[90] = 4977;
	Dimmer_Log[57] = 498;		Dimmer_Log[74] = 1630;		Dimmer_Log[91] = 5337;
	Dimmer_Log[58] = 534;		Dimmer_Log[75] = 1748;		Dimmer_Log[92] = 5722;
	Dimmer_Log[59] = 572;		Dimmer_Log[76] = 1874;		Dimmer_Log[93] = 6136;
	Dimmer_Log[60] = 614;		Dimmer_Log[77] = 2009;		Dimmer_Log[94] = 6579;
	Dimmer_Log[61] = 658;		Dimmer_Log[78] = 2154;		Dimmer_Log[95] = 7055;
	Dimmer_Log[62] = 706;		Dimmer_Log[79] = 2310;		Dimmer_Log[96] = 7565;
	Dimmer_Log[63] = 757;		Dimmer_Log[80] = 2477;		Dimmer_Log[97] = 8111;
	Dimmer_Log[64] = 811;		Dimmer_Log[81] = 2656;		Dimmer_Log[98] = 8698;
	Dimmer_Log[65] = 870;		Dimmer_Log[82] = 2848;		Dimmer_Log[99] = 9326;
	Dimmer_Log[66] = 933;		Dimmer_Log[83] = 3054;		Dimmer_Log[100] = 10000;
	Dimmer_Log[67] = 1000	;	Dimmer_Log[84] = 3275;	


	// Berechnung der PWM Werte

	for(i=0; i<=100; i++)
	{
		Dimmer_PWM[i] = Dimmer_Log[i] * Timer1_Top / 11111;
	}


	// Min/Max Werte definieren
	Dimmer_Max[0] = 100;
	Dimmer_Max[1] = 100;
	Dimmer_Max[2] = 100;
	Dimmer_Max[3] = 100;
	
	Dimmer_Min[0] = 0;
	Dimmer_Min[1] = 0;
	Dimmer_Min[2] = 0;
	Dimmer_Min[3] = 0;
	
	for(i=0; i<4; i++)
	{
		Dimmer_Faktor[i] = (Dimmer_Max[i] - Dimmer_Min[i]) / 100;
	}

}


//-------------------------------------------------------------------------------------------------------
// Dimmer Sollwert vorgeben
// Dimmer_Nr = 0-3; Sollwert = 0-100
void F_Dimmer_Soll(uint8_t Dimmer_Nr, uint8_t Sollwert)
{
	Dimmer_Soll[Dimmer_Nr] = Sollwert;
	Dimmer_Soll_MM[Dimmer_Nr] = Dimmer_Soll[Dimmer_Nr] * Dimmer_Faktor[Dimmer_Nr] + Dimmer_Min[Dimmer_Nr];
}
