//-------------------------------------------------------------------------------------------------------
//
// Funktionen.c
//
//	- Timer abgelaufen
//	- Ausgnge ber CAN schalten
//	- Initialisierungsfunktionen
//	- Automatikfunktionen
//	- Eingang bettigt
//
//-------------------------------------------------------------------------------------------------------


#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "DE.h"
#include "Module.h"



//-------------------------------------------------------------------------------------------------------
// Timer abgelaufen

void F_Timer_abgelaufen(uint8_t Timer_Nr)
{
	switch(Timer_Nr)
	{	
		// Timer fr Module.c
		
		case 0: // Timer 0: Module.c | F_Stromstoss_ein/aus | DA1
				RESET(DA1);
				break;


		case 1: // Timer 1: Module.c | F_Stromstoss_ein/aus | DA2
				RESET(DA2);
				break;


		case 2: // Timer 2: Module.c | F_Stromstoss_ein/aus | DA3
				RESET(DA3);
				break;


		case 3: // Timer 3: Module.c | F_Stromstoss_ein/aus | DA4
				RESET(DA4);
				break;


		case 4: // Timer 4: Module.c | F_Stromstoss_ein/aus | DA5
				RESET(DA5);
				break;


		case 5: // Timer 5: Module.c | F_Stromstoss_ein/aus | DA6
				RESET(DA6);
				break;


		case 6: // Timer 6: Module.c | F_Stromstoss_ein/aus | DA7
				RESET(DA7);
				break;


		case 7: // Timer 7: Module.c | F_Stromstoss_ein/aus | DA8
				RESET(DA8);
				break;


		case 8: // Timer 8: 
				
				break;
				
				
		case 9: //Timer 9: Funktionen.c | F_DE_betaetigt
				// Rollo 1 stoppen
				RESET(Rollo_1_DA_ab);
				RESET(Rollo_1_DA_auf);
				Rollo_Status[1] = 0;
				break;


		case 10: //Timer 10: Funktionen.c | F_DE_betaetigt
				// Rollo 0 stoppen
				RESET(Rollo_0_DA_ab);
				RESET(Rollo_0_DA_auf);
				Rollo_Status[0] = 0;
				break;
		

		// spezifische Timer
		
		case 20:	// Timer 20: Funktionen.c | F_DE_betaetigt | DE1
					F_Dimmer_Soll(0, 0);		// WZ-LI ausschalten
					break;
		
		case 21:	// Timer 21: Funktionen.c | F_DE_betaetigt | DE4			
					F_Dimmer_Soll(0, 0);		// WZ-LI ausschalten
					break;


		default: break;
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////




//-------------------------------------------------------------------------------------------------------
// Ausgnge ber CAN ausschalten

void F_Ausgang_ausschalten(uint8_t Ausgang)
{
	switch(Ausgang)
	{
		// Ausgang nicht dimmbar: RESET(DAx);
		// Ausgang mit Dimmer: F_Dimmer_Soll(Dimmer-Nr, 0);
		// Ausgang mit Stromstoschalter: F_Stromstoss_aus(Nr);
		case 1: F_Dimmer_Soll(0, 0);
				break;
		case 2: RESET(DA2);
				break;
		case 3: RESET(DA3);
				break;
		case 4: RESET(DA4);
				break;
		case 5: RESET(DA5);
				break;
		case 6: F_Stromstoss_aus(6);
				break;
		case 7: F_Stromstoss_aus(7);
				break;
		case 8: F_Stromstoss_aus(8);
				break;
		default: break;
	}
}



//-------------------------------------------------------------------------------------------------------
// Ausgnge ber CAN einschalten

void F_Ausgang_einschalten(uint8_t Ausgang, uint8_t Dimmwert)
{
	switch(Ausgang)
	{
		// Ausgang nicht dimmbar: SET(DAx);
		// Ausgang mit Dimmer: F_Dimmer_Soll(Dimmer-Nr, Sollwert);
		// Ausgang mit Stromstoschalter: F_Stromstoss_ein(Nr);
		case 1:	F_Dimmer_Soll(0, Dimmwert);
				break;
		case 2: SET(DA2);
				break;
		case 3: SET(DA3);
				break;
		case 4: SET(DA4);
				break;
		case 5: SET(DA5);
				break;
		case 6: F_Stromstoss_ein(6);
				break;
		case 7: F_Stromstoss_ein(7);
				break;
		case 8: F_Stromstoss_ein(8);
				break;
		default: break;
	}
}





/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// Initialisierungsfunktionen

void F_Init(void)
{
	F_Stromstoss_ein(6);	// WZ-SD-IT West einschalten
	F_Stromstoss_ein(7);	// WZ-SD-Media einschalten
	F_Stromstoss_ein(8);	// WZ-SD-TV einschalten
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////




//-------------------------------------------------------------------------------------------------------
// Automatikfunktionen

void F_Auto(void)
{
	static uint8_t Minute_alt;
	static uint8_t Sekunde_alt;
	
	tCAN message;				// zum Senden
	message.id = CAN_Adresse;
	message.header.rtr = 0;
	
	
	
	// mintliche Ausfhrung
	if(Minute != Minute_alt  && Sekunde == CAN_Adresse)
	{
		Minute_alt = Minute;
		
		//---------------------------------
		// CON Status ausschicken
		message.header.length = 7;
		message.data[0] = 99;
		message.data[1] = 21;
		message.data[2] = ((DE_Status & 0x0006) >> 1) | ((DE_Status & 0x0300) >> 6) | ((DE_Status & 0x2000) >> 9);
		message.data[3] = WZ_PM_Bewegung;
		message.data[4] = Dimmer_Soll[0];
		message.data[5] = (Rollo_Pos[0] - 10000) / 1000;
		message.data[6] = (Rollo_Pos[1] - 10000) / 1000;
		
		// Antwort senden
		F_MCP2515_tel_senden(&message);
		
		
		//---------------------------------
		// Prsenzmelder: letzte Bewegung
		if(WZ_PM_Bewegung <= 254)
			WZ_PM_Bewegung++;
	}
	
	
	
	
	// sekndliche Ausfhrung
	if(Sekunde != Sekunde_alt)
	{
		Sekunde_alt = Sekunde;
		
		//---------------------------------
		// PM gesperrt
		if(WZ_PM_gesperrt > 0 && WZ_PM_gesperrt < 255)
			WZ_PM_gesperrt--;
	}
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////




//-------------------------------------------------------------------------------------------------------
// Eingang bettigt

void F_DE_betaetigt(void)
{
	static uint8_t M_Dimmer_1, M_Dimmer_2, M_Dimmer_3;
	static uint8_t M_Rollo_1, M_Rollo_2, M_Rollo_3, M_Rollo_4;
				
	// CAN Nachricht erzeugen
	/*tCAN message;       // zum Senden
	
	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 8;
  
	for(i=0; i<8; i++)  // alle Datenbytes auf 0 setzen
	{
		message.data[i] = 0;
	}*/

	// Eingangsauswertung

	// if(F_DE_1kurz(0x0001)) {}
	// if(F_DE_2kurz(0x0001)) {}
	// if(F_DE_lang(0x0001)) {}
	// if(F_DE_fla_fall(0x0001) {}
	// if(F_DE_fla_steig(0x0001) {}
	// if(F_DE_repeat(0x0001) {}
	// bei repeat/lang mssen die Eingnge in der ISR eingetragen werden

	
	// DE1 - WZ-PM
	if(F_DE_fla_steig(DE1))
	{
		if(Stunde < 6 || Stunde > 20)
		{
			if(WZ_PM_gesperrt == 0)
			{
				F_Dimmer_Soll(0, 100);						// WZ-LI einschalten
				F_Timer_starten(20, 0, 2, 0, 0, 0, 0, 0);
				WZ_PM_Bewegung = 0;							// Zhler zurck setzen
				WZ_PM_ein = 1;
			}
		}
	}
	
	
	// DE4 - WZ-TA Haupt
	if(F_DE_1kurz(DE4))
	{		
		if(WZ_PM_ein == 1)
		{
			// wenn PM schon Licht eingeschalten hat, wird nun das auto. ausschalten gesperrt
			WZ_PM_gesperrt = 255;
			F_Timer_starten(24, 4, 0, 0, 0, 0, 0, 0);
			WZ_PM_ein = 0;
		}
		
		else
		{
			if(Dimmer_Status[0] == 0 || (Dimmer_Status[0] == 11 && Dimmer_Soll[0] == 0))
			{
				F_Dimmer_Soll(0, 100);		// WZ-LI einschalten
			}
			
			else if(Dimmer_Status[0] > 0)
			{
				F_Dimmer_Soll(0, 0);		// WZ-LI ausschalten
				WZ_PM_gesperrt = 10;		// WZ-PM wird fr 10 Sekunden nicht beachtet
			}
		}
	}
	
	if(F_DE_repeat(DE4))
	{
		if(M_Dimmer_1 == 0)		// dimmen starten
		{
			M_Dimmer_1 = 1;
			
			if(Dimmer_Ist[0] <= Dimmer_Min[0])
				M_Dimmer_2 = 1;		// rauf dimmen
			
			else
				M_Dimmer_2 = 2;		// runter dimmen
			
			M_Dimmer_3 = Dimmer_Ist[0];
		}
		
		if(M_Dimmer_2 == 1)		// rauf dimmen
		{
			M_Dimmer_3++;
			if(M_Dimmer_3 > 100)
			{
				M_Dimmer_3 = 100;
				M_Dimmer_2 = 2;  // runter dimmen
			}
			F_Dimmer_Soll(0, M_Dimmer_3);
		}
		
		else if(M_Dimmer_2 == 2)	// runter dimmen
		{
			if(M_Dimmer_3 >= Dimmer_Min[0] + 5)
				M_Dimmer_3--;
			
			else
			{
				M_Dimmer_3 = Dimmer_Min[0];
				M_Dimmer_2 = 1;	// rauf dimmen
			}
			
			F_Dimmer_Soll(0, M_Dimmer_3);
		}
	}
	
	if(F_DE_fla_fall(DE4))
	{
		if(M_Dimmer_1 == 1)
		{
			M_Dimmer_1 = 0;
		}
	}
	
	
	// DE5 - WZ-TA unten links
	if(F_DE_1kurz(DE5))
	{	
		if(Rollo_Status[1] == 0)	// Rollo steht
		{
			F_Rollo_ab(1);
			F_Timer_starten(9, 0, 0, 22, 0, 0, 0, 0);	// 22 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(1);
			F_Timer_stopp(9);
		}
	}
	
	if(F_DE_repeat(DE5))
	{
		if(Rollo_Status[1] == 0)	// Rollo steht
		{
			F_Rollo_ab(1);
			M_Rollo_1 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE5))
	{
		if(M_Rollo_1 == 1)
		{
			M_Rollo_1 = 0;
			F_Rollo_stopp(1);
		}
	}
	
	
	// DE6 - WZ-TA oben links
	if(F_DE_1kurz(DE6))
	{
		if(Rollo_Status[1] == 0)	// Rollo steht
		{
			F_Rollo_auf(1);
			F_Timer_starten(9, 0, 0, 22, 0, 0, 0, 0);	// 22 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(1);
			F_Timer_stopp(9);
		}
	}
	
	if(F_DE_repeat(DE6))
	{
		if(Rollo_Status[1] == 0)	// Rollo steht
		{
			F_Rollo_auf(1);
			M_Rollo_2 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE6))
	{
		if(M_Rollo_2 == 1)
		{
			M_Rollo_2 = 0;
			F_Rollo_stopp(1);
		}
	}
	
	
	// DE7 - WZ-TA oben rechts
	if(F_DE_1kurz(DE7))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_ab(0);
			F_Timer_starten(10, 0, 0, 22, 0, 0, 0, 0);	// 22 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(0);
			F_Timer_stopp(10);
		}
	}
	
	if(F_DE_repeat(DE7))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_ab(0);
			M_Rollo_3 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE7))
	{
		if(M_Rollo_3 == 1)
		{
			M_Rollo_3 = 0;
			F_Rollo_stopp(0);
		}
	}
	
	
	// DE8 - WZ-TA unten rechts
	if(F_DE_1kurz(DE8))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_auf(0);
			F_Timer_starten(10, 0, 0, 22, 0, 0, 0, 0);	// 22 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(0);
			F_Timer_stopp(10);
		}
	}
	
	if(F_DE_repeat(DE8))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_auf(0);
			M_Rollo_4 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE8))
	{
		if(M_Rollo_4 == 1)
		{
			M_Rollo_4 = 0;
			F_Rollo_stopp(0);
		}
	}
}

