//-------------------------------------------------------------------------------------------------------
//
// Funktion bei Meldung Strom-Aus
//
//-------------------------------------------------------------------------------------------------------


#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "Stromaus.h"
#include "Module.h"


//-------------------------------------------------------------------------------------------------------
// Meldung Stromausfall
void F_Stromaus(void)
{
	uint8_t i=0;
	uint8_t j=0;
	
	// CAN Nachricht erzeugen
	tCAN message_rec;			// zum Empfangen
	
	TIMSK0 &= ~(1<<TOIE0);	// Timer 0 stoppen
	TIMSK1 &= ~(1<<TOIE1);	// Timer 1 stoppen
	RESET(LED_G);
	SET(LED_R);
	
	F_EEPROM_schreiben(0, 0, 1);  // Strom Aus Merker setzen
	
	F_EEPROM_schreiben(0, 10, Minute);
	F_EEPROM_schreiben(0, 11, Stunde);
	F_EEPROM_schreiben(0, 12, Tag);
	F_EEPROM_schreiben(0, 13, Monat);
	F_EEPROM_schreiben(0, 14, Jahr);
	
	F_EEPROM_schreiben(0, 20, PIND);			// Status Ausgnge
	F_EEPROM_schreiben(0, 21, Dimmer_Soll[0]);	// Dimmer 1 Sollwert
	F_EEPROM_schreiben(0, 22, Dimmer_Soll[1]);	// Dimmer 2 Sollwert
	F_EEPROM_schreiben(0, 23, Dimmer_Soll[2]);	// Dimmer 3 Sollwert
	F_EEPROM_schreiben(0, 24, Dimmer_Soll[3]);	// Dimmer 4 Sollwert
	
	// Ausgnge rcksetzen
	RESET(DA1);
	RESET(DA2);
	RESET(DA3);
	RESET(DA4);
	RESET(DA5);
	RESET(DA6);
	RESET(DA7);
	RESET(DA8);
	
	// aktive Timer speichern
	for(i=0; i<Anzahl_Timer; i++)
	{
		if(Timer_Status[i] == 1)  // Timer (i+1) ist aktiv
		{
			F_EEPROM_schreiben(0, 30 + j, i + 1);
			F_EEPROM_schreiben(0, 31 + j, *T_End_Jahr);
			F_EEPROM_schreiben(0, 32 + j, *T_End_Monat);
			F_EEPROM_schreiben(0, 33 + j, *T_End_Tag);
			F_EEPROM_schreiben(0, 34 + j, *T_End_Stunde);
			F_EEPROM_schreiben(0, 35 + j, *T_End_Minute);
			F_EEPROM_schreiben(0, 36 + j, *T_End_Sekunde);
			F_EEPROM_schreiben(0, 37 + j, *T_End_10ms);
			
			j += 8;
		}
	}
	
	j = j / 8;  // Anzahl der gespeicherten Timer
		F_EEPROM_schreiben(0, 29, j);
	
	
	while(1)  // auf Stromaus oder Softwarereset warten
	{
		// CAN Telegramm empfangen
		if(MCP2515_tel_empfangen > 0)  // CAN Telegramm empfangen
		{
			F_MCP2515_tel_empfangen(&message_rec);
			MCP2515_tel_empfangen--;

			if(message_rec.data[0] == CAN_Adresse || message_rec.data[0] == 99)  // Prfen, ob die Adresse passt
			{
				if(message_rec.data[1] == 70)		// Steuerbefehle
				{
					if(message_rec.data[2] == 1)	// Softwarereset
					{
						cli();
						wdt_enable(WDTO_15MS);
						while(1);
					}
				}
			}
		}
	}
}


//-------------------------------------------------------------------------------------------------------
// Initialisiern nach Stromausfall
void F_Stromaus_init(void)
{
	uint8_t i = 0;
	uint8_t j = 0;
	uint8_t k = 0;
	uint8_t Stromaus_Stunde;
	uint8_t Stromaus_Minute;
	uint8_t Stromaus_Jahr;
	uint8_t Stromaus_Monat;
	uint8_t Stromaus_Tag;
	uint8_t Unterschied_Tag = 0;
	uint8_t Unterschied_Stunde;
	uint8_t Unterschied_Minute;
	uint8_t Status_DA;
	
	
	// CAN Nachricht erzeugen
	tCAN message_rec;			// zum Empfangen
	
	
	// 2s warten auf aktuelle Uhrzeit von IH_EGW
	i = 0;
	while(i < 200)
	{
		// CAN Telegramm empfangen
		if(MCP2515_tel_empfangen > 0)  // CAN Telegramm empfangen
		{
			F_MCP2515_tel_empfangen(&message_rec);
			MCP2515_tel_empfangen--;

			if(message_rec.data[0] == CAN_Adresse || message_rec.data[0] == 99)  // Prfen, ob die Adresse passt
			{
				if(message_rec.data[1] == 80)  // Uhrzeit-Telegramm
				{
					Stunde = message_rec.data[2];
					Minute = message_rec.data[3];
					Sekunde = message_rec.data[4];
					TCNT1 = 40536;
					Ms10 = 0;
					Tag = message_rec.data[5];
					Wochentag = message_rec.data[6] & 0x0F;
					Monat = (message_rec.data[6] & 0xF0) >> 4;
					Jahr = message_rec.data[7];
					
					i = 210;
				}
			}
		}
		
		_delay_ms(10);
		i++;
	}
	
	
	// keine Uhrzeit von IH_EGW	
	if(i == 200)
	{
		Stunde = 0;
		Minute = 0;
		Sekunde = 0;
		TCNT1 = 40536;
		Ms10 = 0;
		Tag = 1;
		Wochentag = 5;
		Monat = 1;
		Jahr = 16;
	}
	
	// Uhrzeit von IH_EGW erhalten
	else 
	{
		// prfen wie viel Zeit seit dem Stromausfall vergangen ist
		Stromaus_Stunde = F_EEPROM_lesen(0, 11);
		Stromaus_Minute = F_EEPROM_lesen(0, 10);
		Stromaus_Jahr = F_EEPROM_lesen(0, 14);
		Stromaus_Monat = F_EEPROM_lesen(0, 13);
		Stromaus_Tag = F_EEPROM_lesen(0, 12);
		
		if(Stromaus_Jahr == Jahr)  // Jahr gleich
		{
			if(Stromaus_Monat == Monat)  // Monat gleich
			{
				if(Stromaus_Tag == Tag)  // Tag gleich
				{
					Unterschied_Tag = 0;
					
					if(Stromaus_Stunde == Stunde)  // Stunde gleich
					{
						Unterschied_Stunde = 0;
						
						if(Stromaus_Minute == Minute)  // Minute gleich
						{
							Unterschied_Minute = 1;					
						}
						else  // Minute nicht gleich
						{
							Unterschied_Minute = Minute - Stromaus_Minute;
						}
					}
					else  // Stunde nicht gleich
					{
						Unterschied_Minute = 60 - Stromaus_Minute + Minute;
						Unterschied_Stunde = Stunde - (Stromaus_Stunde + 1);
					}
				}
				else  // Tag nicht gleich
				{
					Unterschied_Minute = 60 - Stromaus_Minute + Minute;
					Unterschied_Stunde = 24 - (Stromaus_Stunde + 1) + Stunde;
					Unterschied_Tag = Tag - (Stromaus_Tag + 1);
				}
			}
			else  // Monat nicht gleich
			{
				Unterschied_Minute = 60 - Stromaus_Minute + Minute;
				Unterschied_Stunde = 24 - (Stromaus_Stunde + 1) + Stunde;
				
				if((Monat - Stromaus_Monat) == 1)
				{
					if(Stromaus_Monat == 1 || Stromaus_Monat == 3 || Stromaus_Monat == 5 || Stromaus_Monat == 7 ||
					Stromaus_Monat == 8 || Stromaus_Monat == 10 || Stromaus_Monat == 12)
					{
						Unterschied_Tag = 31 - (Stromaus_Tag + 1) + Tag;
					}
					
					else if(Stromaus_Monat == 4 || Stromaus_Monat == 6 || Stromaus_Monat == 9 || Stromaus_Monat == 11)
					{
						Unterschied_Tag = 30 - (Stromaus_Tag + 1) + Tag;
					}
					
					else if(Stromaus_Monat == 2)
					{
						if(Jahr%4 == 0)  // Schaltjahr
						Unterschied_Tag = 29 - (Stromaus_Tag + 1) + Tag;
						else
						Unterschied_Tag = 28 - (Stromaus_Tag + 1) + Tag;
					}
				}
				else  // Stromaus lnger als 1 Monat her
				{
					Unterschied_Tag = 100;  
				}

			}
		}
		else  // Jahr nicht gleich
		{
			if(Stromaus_Monat == 12 && Monat == 1)  // Stromaus direkt ber Silvester
			{
				Unterschied_Minute = 60 - Stromaus_Minute + Minute;
				Unterschied_Stunde = 24 - (Stromaus_Stunde + 1) + Stunde;
				Unterschied_Tag = 31 - (Stromaus_Tag + 1) + Tag; 
			}
			
			else
			{
				Unterschied_Minute = 0;
				Unterschied_Stunde = 0;
				Unterschied_Tag = 100;  // Stromaus lnger als 1 Monat her
			}
		}
		
		
		
		// Aktionen nach Strom zurck
		
		if(Unterschied_Tag == 0 && Unterschied_Stunde == 0 && Unterschied_Minute <= 30)
		{
			// gespeicherte Timer auslesen
			k = 0;
			for(i=0; i<F_EEPROM_lesen(0,29); i++)
			{
				j = F_EEPROM_lesen(0, 30 + k);					// Timer Nummer
				T_End_Jahr[j] = F_EEPROM_lesen(0, 31 + k);		// Jahr
				T_End_Monat[j] = F_EEPROM_lesen(0, 32 + k);		// Jahr
				T_End_Tag[j] = F_EEPROM_lesen(0, 33 + k);		// Jahr
				T_End_Stunde[j] = F_EEPROM_lesen(0, 34 + k);	// Jahr
				T_End_Minute[j] = F_EEPROM_lesen(0, 35 + k);	// Jahr
				T_End_Sekunde[j] = F_EEPROM_lesen(0, 36 + k);	// Jahr
				T_End_10ms[j] = F_EEPROM_lesen(0, 37 + k);		// Jahr
				Timer_Status[j] = 1;							// Timer einschalten
				
				k += 8;
			}
			
			
			// Ausgnge und Dimmer setzen
			Dimmer_Soll[0] = F_EEPROM_lesen(0, 21);
			Dimmer_Soll[1] = F_EEPROM_lesen(0, 22);
			Dimmer_Soll[2] = F_EEPROM_lesen(0, 23);
			Dimmer_Soll[3] = F_EEPROM_lesen(0, 24);
			
			Status_DA = F_EEPROM_lesen(0, 20);
			
			// fr alle Ausgnge ohne Dimmer
			if((Status_DA & 0x01) >= 1)
				SET(DA1);
			if((Status_DA & 0x02) >= 1)
				SET(DA2);
			if((Status_DA & 0x04) >= 1)
				SET(DA3);
			if((Status_DA & 0x08) >= 1)
				SET(DA4);
			if((Status_DA & 0x10) >= 1)
				SET(DA5);
			if((Status_DA & 0x20) >= 1)
				SET(DA6);
			if((Status_DA & 0x40) >= 1)
				SET(DA7);
			if((Status_DA & 0x80) >= 1)
				SET(DA8);
		}		
	}
}