//-------------------------------------------------------------------------------------------------------
//
// Timer.c
//
//-------------------------------------------------------------------------------------------------------


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "Config.h"
#include "SPI.h"
#include "EEPROM.h"
#include "DE.h"
#include "Funktionen.h"
#include "Dimmer.h"


//-------------------------------------------------------------------------------------------------------
// Timer starten
// F_Timer_starten(1, 0, 0, 10, 0, 0, 0, 0); -> Timer 1: 10 Sekunden

void F_Timer_starten(uint8_t T_Nr, uint8_t T_St, uint8_t T_Mi, uint8_t T_Se, uint8_t T_10, 
uint8_t T_Ta, uint8_t T_Mo, uint8_t T_Ja)
{
	  // Timer Endwerte speichern

	  T_End_Stunde[T_Nr] = Stunde + T_St;
	  T_End_Minute[T_Nr] = Minute + T_Mi;
	  T_End_Sekunde[T_Nr] = Sekunde + T_Se;
	  T_End_10ms[T_Nr] = Ms10 + T_10;
	  T_End_Tag[T_Nr] = Tag + T_Ta;
	  T_End_Monat[T_Nr] = Monat + T_Mo;
	  T_End_Jahr[T_Nr] = Jahr + T_Ja;

	  if(T_End_10ms[T_Nr] >= 100)
	  {
			T_End_10ms[T_Nr] -= 100;
			T_End_Sekunde[T_Nr]++;
	  }

	  if(T_End_Sekunde[T_Nr] >= 60)
	  {
			T_End_Sekunde[T_Nr] -= 60;
			T_End_Minute[T_Nr]++;
	  }

	  if(T_End_Minute[T_Nr] >= 60)
	  {
			T_End_Minute[T_Nr] -= 60;
			T_End_Stunde[T_Nr]++;
	  }

	  if(T_End_Stunde[T_Nr] >= 24)
	  {
			T_End_Stunde[T_Nr] -= 24;
			T_End_Tag[T_Nr]++;
	  }

	  if(Monat == 1 || Monat == 3 || Monat == 5 || Monat == 7 || Monat == 8 || Monat == 10 || Monat == 12)
	  {
			if(T_End_Tag[T_Nr] > 31)
			{
				T_End_Tag[T_Nr] -= 31;
				T_End_Monat[T_Nr]++;
			}
	  }
	  else if(Monat == 4 || Monat == 6 || Monat == 9 || Monat == 11)
	  {
			if(T_End_Tag[T_Nr] > 30)
			{
				T_End_Tag[T_Nr] -= 30;
				T_End_Monat[T_Nr]++;
			}
	  }
	  else if(Monat == 2)
	  {
			if(Jahr%4 == 0)  // Schaltjahr
			{
				  if(T_End_Tag[T_Nr] > 29)
				  {
					  T_End_Tag[T_Nr] -= 29;
					  T_End_Monat[T_Nr]++;
				  }
			}
			else
			{
				  if(T_End_Tag[T_Nr] > 28)
				  {
					  T_End_Tag[T_Nr] -= 28;
					  T_End_Monat[T_Nr]++;
				  }
			}
	  }

	  if(T_End_Monat[T_Nr] > 12)
	  {
			T_End_Monat[T_Nr] -= 12;
			T_End_Jahr[T_Nr]++;
	  }
	  
	  
	  Timer_Status[T_Nr] = 1;  // Timer starten
}


//-------------------------------------------------------------------------------------------------------
// laufenden Timer stoppen und zurcksetzen

void F_Timer_stopp(uint8_t T_Nr)
{
	Timer_Status[T_Nr] = 0;
}



//-------------------------------------------------------------------------------------------------------
// Timer abgelaufen?

void F_Timer(void)
{
	  uint8_t i;

	  // Timer

	  for(i=0; i<Anzahl_Timer; i++)
	  {
			if(Timer_Status[i] == 1)
			{
				  if((Jahr >= T_End_Jahr[i]) && (Monat >= T_End_Monat[i]) && (Tag >= T_End_Tag[i]) &&
				  (Stunde >= T_End_Stunde[i]) && (Minute >= T_End_Minute[i]) && 
				  (Sekunde >= T_End_Sekunde[i]) && (Ms10 >= T_End_10ms[i]))
				  {
						Timer_Status[i] = 0;  // Timer abgelaufen

						switch(i)
						{
							case 0: // Timer 0: Funktionen.c | F_Stromstoss_ein/aus | DA6
									RESET(DA6);
									break;


							case 1: // Timer 1: Funktionen.c | F_Stromstoss_ein/aus | DA7
									RESET(DA7);
									break;


							case 2: // Timer 2: Funktionen.c | F_Stromstoss_ein/aus | DA8
									RESET(DA8);
									break;


							case 3: // Timer 3: Funktionen.c | F_Rollo_init
									// Rollo Sd stoppen
									RESET(DA5);
									RESET(DA4);

									// Rollo Ost stoppen
									RESET(DA3);
									RESET(DA2);

									Rollo_Status_S = 0;
									Rollo_Status_O = 0;		
									break;
							
							
							case 4:	// Timer 4: DE.c | F_DE_betaetigt | DE1
									F_Dimmer_Soll(0, 0);		// WZ-LI ausschalten
									break;


							default: break;
						}
				  }
			}
	  }
}

