//-------------------------------------------------------------------------------------------------------
//
// Fehlerauswertung
//
//-------------------------------------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>

#include "MCP2515.h"
#include "Error.h"
#include "config.h"
#include "EEPROM.h"
#include "udp.h"
#include "enc28j60.h"
#include "stack.h"


//-------------------------------------------------------------------------------------------------------
// Fehler initialisierung
void F_Error_init(void)
{
	uint8_t i;
	
	// Puffer leeren
	for(i=0; i<10; i++)
	{
		Error_Puffer[i] = 0;
	}

	Error_Stand = 0;
}


//-------------------------------------------------------------------------------------------------------
// Fehlerausgabe
void F_Error_Ausgabe(void)
{
	static uint8_t Status;      // 0: Zehnerstelle wird angezeigt
	// 1: Einerstelle wird angezeigt
	// 2: Warten
	static uint8_t Z_Zaehler;  // Zhlvariable Zehnerstelle
	static uint8_t E_Zaehler;  // Zhlvariable Einerstelle
	static uint8_t P_Zaehler;  // Zhlvariable Pause
	static uint8_t Ein_Aus;    // Led ein oder aus
	static uint8_t Pufferzaehler;  // zeigt die Position im Puffer an, die Ausgegeben wird
	static uint8_t neuerFehler;  // 0 wenn eine neue Pufferposition ausgegeben werden kann

	if(Error_Stand > 0)  // Fehler zum Anzeigen
	{
		if(neuerFehler == 0)  // neuer Fehler wird angezeigt
		{
			Z_Zaehler = Error_Puffer[Pufferzaehler] / 10;
			E_Zaehler = Error_Puffer[Pufferzaehler] % 10;
			neuerFehler = 1;

			if(Z_Zaehler == 0)
			Status = 1;
			
			if(Pufferzaehler < (Error_Stand - 1))
			Pufferzaehler++;

			else if(Pufferzaehler == (Error_Stand-1))
			Pufferzaehler = 0;
		}

		if(Status == 0) // Zehnerstelle ausgeben
		{
			if(Ms10 == 0)  // 0ms
			{
				if(Z_Zaehler > 0 && Ein_Aus == 0)
				{
					RESET(LED_G);
					SET(LED_R);
					Z_Zaehler--;
					Ein_Aus = 1;
				}
			}
			
			if(Ms10 == 50 && Ein_Aus == 1)  // 500ms
			{
				RESET(LED_R);
				Ein_Aus = 0;

				if(Z_Zaehler == 0)
				{
					Status = 1;

					if(E_Zaehler == 0)  // keine Einerstelle zum Ausgeben
					Status = 2;
				}
			}
		}

		else if(Status == 1) // Einerstelle ausgeben
		{
			if(Ms10 == 0)
			{
				if(E_Zaehler > 0 && Ein_Aus == 0)  // 0ms
				{
					SET(LED_G);
					E_Zaehler--;
					Ein_Aus = 1;
				}
			}
			
			if(Ms10 == 50 && Ein_Aus == 1)  // 500ms
			{
				RESET(LED_G);
				Ein_Aus = 0;

				if(E_Zaehler == 0)
				Status = 2;
			}
		}

		else  // Pause
		{
			if(Ms10 == 0 && Ein_Aus == 0)  // 0ms
			{
				P_Zaehler++;
				Ein_Aus = 1;

				if(P_Zaehler == 2)
				{
					Status = 0;
					P_Zaehler = 0;
					neuerFehler = 0;
				}
			}
			else if(Ms10 == 50 && Ein_Aus == 1)  // 500ms
			Ein_Aus = 0;
		}
	}
}


//-------------------------------------------------------------------------------------------------------
// neuer Fehler
void F_Error(uint8_t Code, uint8_t Info_1, uint8_t Info_2, uint8_t Info_3, uint8_t Info_4, uint8_t Info_5)
{
	uint8_t i;
	tCAN message;			// Telegramm zum Senden

	// CAN Fehlertelegramm fllen
	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 8;
	message.data[0] = 99;
	message.data[1] = 90;
	message.data[2] = Code;
	message.data[3] = Info_1;
	message.data[4] = Info_2;
	message.data[5] = Info_3;
	message.data[6] = Info_4;
	message.data[7] = Info_5;

		
	// Fehlertelegramm per Ethernet verschicken
	eth_buffer[UDP_DATA_START] = 66;	// Telegrammtyp B
	eth_buffer[UDP_DATA_START+1] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+2] = 48;	// CAN Adresse Sender = 9
	eth_buffer[UDP_DATA_START+3] = 48;
	eth_buffer[UDP_DATA_START+4] = 57;
	eth_buffer[UDP_DATA_START+5] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+6] = F_InttoChar_Hunderter(message.data[0]);	// CAN Byte 0
	eth_buffer[UDP_DATA_START+7] = F_InttoChar_Zehner(message.data[0]);
	eth_buffer[UDP_DATA_START+8] = F_InttoChar_Einer(message.data[0]);
	eth_buffer[UDP_DATA_START+9] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+10] = F_InttoChar_Hunderter(message.data[1]);	// CAN Byte 1
	eth_buffer[UDP_DATA_START+11] = F_InttoChar_Zehner(message.data[1]);
	eth_buffer[UDP_DATA_START+12] = F_InttoChar_Einer(message.data[1]);
	eth_buffer[UDP_DATA_START+13] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+14] = F_InttoChar_Hunderter(message.data[2]);	// CAN Byte 2
	eth_buffer[UDP_DATA_START+15] = F_InttoChar_Zehner(message.data[2]);
	eth_buffer[UDP_DATA_START+16] = F_InttoChar_Einer(message.data[2]);
	eth_buffer[UDP_DATA_START+17] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+18] = F_InttoChar_Hunderter(message.data[3]);	// CAN Byte 3
	eth_buffer[UDP_DATA_START+19] = F_InttoChar_Zehner(message.data[3]);
	eth_buffer[UDP_DATA_START+20] = F_InttoChar_Einer(message.data[3]);
	eth_buffer[UDP_DATA_START+21] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+22] = F_InttoChar_Hunderter(message.data[4]);	// CAN Byte 4
	eth_buffer[UDP_DATA_START+23] = F_InttoChar_Zehner(message.data[4]);
	eth_buffer[UDP_DATA_START+24] = F_InttoChar_Einer(message.data[4]);
	eth_buffer[UDP_DATA_START+25] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+26] = F_InttoChar_Hunderter(message.data[5]);	// CAN Byte 5
	eth_buffer[UDP_DATA_START+27] = F_InttoChar_Zehner(message.data[5]);
	eth_buffer[UDP_DATA_START+28] = F_InttoChar_Einer(message.data[5]);
	eth_buffer[UDP_DATA_START+29] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+30] = F_InttoChar_Hunderter(message.data[6]);	// CAN Byte 6
	eth_buffer[UDP_DATA_START+31] = F_InttoChar_Zehner(message.data[6]);
	eth_buffer[UDP_DATA_START+32] = F_InttoChar_Einer(message.data[6]);
	eth_buffer[UDP_DATA_START+33] = 59;	// Trennzeichen ;
	eth_buffer[UDP_DATA_START+34] = F_InttoChar_Hunderter(message.data[7]);	// CAN Byte 7
	eth_buffer[UDP_DATA_START+35] = F_InttoChar_Zehner(message.data[7]);
	eth_buffer[UDP_DATA_START+36] = F_InttoChar_Einer(message.data[7]);
	eth_buffer[UDP_DATA_START+37] = 33;	// Endzeichen !
	
	create_new_udp_packet(38, 7600, 7600, 0x0A00A8C0);  // 192.168.0.10
	
	
	//Fehlertelegramm per CAN verschicken
	// MCP2515 initialisierung
	if(F_MCP2515_init() != 0)
	{
		// initialisieren mglich
		while(!F_MCP2515_freier_puffer())
		{
			// auf freien Puffer warten
		}
		F_MCP2515_tel_senden(&message);
	}
	
	// Fehlerpuffer
	for(i=0; i<10; i++)
	{
		if(Error_Puffer[i] == 0)  // ersten freien Puffer beschreiben
		{
			Error_Puffer[i] = Code;
			i=10;
			Error_Stand++;
		}
		
		else if(Error_Puffer[i] == Code)  // Fehlercode bereits im Puffer
		i=10;
	}
}