//-------------------------------------------------------------------------------------------------------
//
// I2C Echtzeituhr - DS1307
//
//-------------------------------------------------------------------------------------------------------


#include <avr/io.h>
#include <avr/interrupt.h>

#include "I2Cmaster.h"
#include "Config.h"
#include "MCP2515.h"
#include "Uhr.h"
#include "enc28j60.h"
#include "stack.h"
#include "Error.h"
#include "enc28j60.h"
#include "stack.h"
#include "udp.h"

//-------------------------------------------------------------------------------------------------------
// globale Variablen
volatile unsigned long time;
volatile unsigned long stack_watchdog;


//-------------------------------------------------------------------------------------------------------
// Uhrzeittimer 10ms
ISR(TIMER1_OVF_vect)
{
	TCNT1 = 40536;

	// 10ms Taktmerker erhhen
	Ms10++;
}


//-------------------------------------------------------------------------------------------------------
// I2C Uhr Takt 1Hz
ISR(INT1_vect)
{
	static uint8_t S_W;
	
	Ms10 = 0;
	TCNT1 = 40536;
	
	// Zeit und Datum
	Sekunde++;
	
	if(Sekunde >= 60)
	{
		Sekunde = 0;
		Minute++;
		
		if(Minute >= 60)
		{
			Minute = 0;
			Stunde++;
			
			if(Stunde >= 24)
			{
				Stunde = 0;
				Tag++;
								
				// 31 Tage
				if(Monat == 1 || Monat == 3 || Monat == 5 || Monat == 7 || Monat == 8 || Monat == 10 || Monat == 12)
				{
					if(Tag > 31)
					{
						Tag = 1;
						Monat++;
					}
				}
				
				// 30 Tage
				else if(Monat == 4 || Monat == 6 || Monat == 9 || Monat == 11)
				{
					if(Tag > 30)
					{
						Tag = 1;
						Monat++;
					}
				}
				
				// 28 oder 29 Tage
				else if(Monat == 2)
				{
					if(Jahr%4 == 0)  // Schaltjahr
					{
						if(Tag > 29)
						{
							Tag = 1;
							Monat++;
						}	
					}
					
					else
					{
						if(Tag > 28)
						{
							Tag = 1;
							Monat++;
						}
					}
				}
				
				
				Wochentag++;
				if(Wochentag > 7)
					Wochentag = 1;
				
				
				if(Monat > 12)
				{
					Monat = 1;
					Jahr++;
				}
			}
		}
	}
	/*
	tCAN message;       // Telegramm zum Senden

	// Uhrzeit und Datum per CAN verschicken
	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 8;
	message.data[0] = 99;
	message.data[1] = 80;
	message.data[2] = Stunde;
	message.data[3] = Minute;
	message.data[4] = Sekunde;
	message.data[5] = Tag;
	message.data[6] = Wochentag | (Monat<<4);
	message.data[7] = Jahr;

	F_MCP2515_tel_senden(&message);
	*/
	
	// Sommer-Winterzeit Umstellung
	if(Monat == 3 && Wochentag == 7  && Tag >= 25)	// W->S: letzter SO im Mrz
	{
		if(Stunde == 2)  // von 2:00 auf 3:00 stellen
		{
			Stunde = 3;
		}
	}
	
	if(Monat == 10 && Wochentag == 7 && Tag >= 25)  // S->W: letzter SO im Oktober
	{
		if(Stunde == 3 && S_W == 0)
		{
			Stunde = 2;
			S_W = 1;
		}
		
		if(Stunde == 4)
		{
			S_W = 0;
		}
	}
	
	
	// Ethernet Fehlerberwachung
	time++;
	if((stack_watchdog++) > WTT)  //emergency reset of the stack
	{
		RESET_STACK();
	}
	eth.timer = 1;
}


//-------------------------------------------------------------------------------------------------------
// Uhr initialisieren
void F_Uhr_init(void)
{
	// externer Interrupt (INT1)
	SET_INPUT(Uhr_INT);
	SET(Uhr_INT);
	EICRA |= (1<<ISC11);	// fallende Flanken
	EIMSK |= (1<<INT1);
	
	// Timer 1: Uhrzeittimer initialisieren
	TCCR1A = 0;
	TCCR1B &= ~((1<<CS12) | (1<<CS10)); // Prescale setzen
	TCCR1B |= (1<<CS11);                // Prescale setzen
	TCNT1 = 40536;
	TIFR1 = (1 << TOV1);				// Overflow-flag lschen
	TIMSK1 |= (1<<TOIE1);
}


//-------------------------------------------------------------------------------------------------------
// Echtzeituhr stellen
void F_Uhr_stellen(void)
{
	uint8_t U_Sek;
	uint8_t U_Min;
	uint8_t U_Std;
	uint8_t U_Tag;
	uint8_t U_Monat;
	uint8_t U_Jahr;

	U_Sek = 0x7F & (((Sekunde / 10)<<4) | (Sekunde % 10));
	U_Min = ((Minute / 10)<<4) | (Minute % 10);
	U_Std = 0x3F & (((Stunde / 10)<<4) | (Stunde % 10));
	U_Tag = ((Tag / 10)<<4) | (Tag % 10);
	U_Monat = ((Monat / 10)<<4) | (Monat % 10);
	U_Jahr = ((Jahr / 10)<<4) | (Jahr % 10);

	i2c_start_wait(0xD0); // Slave bereit zum schreiben?
	i2c_write(0x00);  // Address 0

	i2c_write(U_Sek);  // Sekunden
	i2c_write(U_Min);  // Minuten
	i2c_write(U_Std);  // Stunden
	i2c_write(Wochentag);  // Wochentag
	i2c_write(U_Tag);  // Tag
	i2c_write(U_Monat);  // Monat
	i2c_write(U_Jahr);  // Jahr
	i2c_write(0x10);  // Control Register: SQWE = 1
	
	i2c_stop();       // Zugriff beenden
}

//-------------------------------------------------------------------------------------------------------
// Uhrzeit und Datum auslesen
void F_Uhr_lesen(void)
{
	uint8_t U_Sek;
	uint8_t U_Min;
	uint8_t U_Std;
	uint8_t U_Tag;
	uint8_t U_Monat;
	uint8_t U_Jahr;

	i2c_start_wait(0xD0); // Slave schreiben
	i2c_write(0x00);
	i2c_stop();
	
	i2c_rep_start(0xD1);  // Slave lesen
	U_Sek = i2c_readAck();
	U_Min = i2c_readAck();
	U_Std = i2c_readAck();
	Wochentag = i2c_readAck();
	U_Tag = i2c_readAck();
	U_Monat = i2c_readAck();
	U_Jahr = i2c_readNak();
	
	i2c_stop();  // Zugriff beenden

	Sekunde = (((0x70 & U_Sek) >> 4) * 10) + (0x0F & U_Sek);
	Minute = (((0x70 & U_Min) >> 4) * 10) + (0x0F & U_Min);
	Stunde = (((0x30 & U_Std) >> 4) * 10) + (0x0F & U_Std);
	Tag = (((0x30 & U_Tag) >> 4) * 10) + (0x0F & U_Tag);
	Monat = (((0x10 & U_Monat) >> 4) * 10) + (0x0F & U_Monat);
	Jahr = (((0xF0 & U_Jahr) >> 4) * 10) + (0x0F & U_Jahr);
}

