/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        03.11.2007
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_	
	
	#include <avr/io.h>
	
	//-------------------------------------------------------------------------------------------------------
	// Ethernet
	
	//ETH_M32_EX (www.ulrichradig.de)
	#define USE_ENC28J60	1
	
	//Konfiguration der PORTS (HEX)
	//1=OUTPUT / 0=INPUT
	#define OUTA 		0x07
	#define OUTC 		0x0F
	#define OUTD 		0x00
	//Achtung!!!! an PORTB ist der ENC
	//nur ndern wenn man wei was man macht!

    //Watchdog timer for the ENC2860, resets the stack if timeout occurs
    #define WTT 1200 //Watchdog timer in timer interrupt

	//Umrechnung von IP zu unsigned long
	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//IP des Webservers und des Routers
	#define MYIP		IP(10,0,0,9)
    #define ROUTER_IP	IP(10,0,0,138)

	//Netzwerkmaske
	#define NETMASK		IP(255,255,255,0)
	
	//MAC Adresse des Webservers	
	#define MYMAC1	0x00
	#define MYMAC2	0x20
	#define MYMAC3	0x18
	#define MYMAC4	0xB1	
	#define MYMAC5	0x15
	#define MYMAC6	0x6F
	
	#define RESET_STACK() {asm("ldi r30,0"); asm("ldi r31,0"); asm("ijmp");}
    

	// NTP
	#define NTP_IP			IP(10,0,0,140)		// NAS
	#define NTP_CLIENT_PORT		2300
	#define NTP_SERVER_PORT		123
	
	#define HTONL(x) ((x & 0xFF000000)>>24)+((x & 0x00FF0000)>>8)+((x & 0x0000FF00)<<8)+((x & 0x000000FF)<<24)


	//-------------------------------------------------------------------------------------------------------
	
	// Makros
	#define	RESET(x)		_XRS(x)
	#define	SET(x)			_XS(x)
	#define	TOGGLE(x)		_XT(x)
	#define	SET_OUTPUT(x)	_XSO(x)
	#define	SET_INPUT(x)	_XSI(x)
	#define	IS_SET(x)		_XR(x)

	#define	PORT(x)			_port2(x)
	#define	DDR(x)			_ddr2(x)
	#define	PIN(x)			_pin2(x)

	#define	_XRS(x,y)		PORT(x) &= ~(1<<y)
	#define	_XS(x,y)		PORT(x) |= (1<<y)
	#define	_XT(x,y)		PORT(x) ^= (1<<y)

	#define	_XSO(x,y)		DDR(x) |= (1<<y)
	#define	_XSI(x,y)		DDR(x) &= ~(1<<y)
	#define	_XR(x,y)		((PIN(x) & (1<<y)) != 0)

	#define	_port2(x)		PORT ## x
	#define	_ddr2(x)		DDR ## x
	#define	_pin2(x)		PIN ## x

	//-------------------------------------------------------------------------------------------------------
	
	// SPI MCP2515
	#define	P_MOSI			B,5
	#define	P_MISO			B,6
	#define	P_SCK			B,7
	#define	MCP2515_SS		B,4

	// Interrupts
	#define	MCP2515_INT		D,2		// INT0
	#define Uhr_INT			D,3		// INT1
	#define ENC28J60_INT	B,2		// INT2

	// Statusled
	#define LED_G			C,2		// grn
	#define LED_R			C,3		// rot
	
	
	//-------------------------------------------------------------------------------------------------------
	// CAN

	#define CAN_Adresse			9
	
	
	//-------------------------------------------------------------------------------------------------------
	// Taktfrequenz
	
	#ifndef F_CPU
	#define F_CPU 20000000UL
	#endif
	
	
	//-------------------------------------------------------------------------------------------------------
	// globale Variablen

	// Uhrzeit und Datum
	volatile uint8_t Ms10;                // 0-99
	volatile uint8_t Sekunde;             // 0-59
	volatile uint8_t Minute;              // 0-59
	volatile uint8_t Stunde;              // 0-23
	volatile uint8_t Tag;                 // 1-31
	volatile uint8_t Monat;               // 1-12
	volatile uint8_t Jahr;                // Wert + 2000 rechnen
	volatile uint8_t Wochentag;           // 1=MO, 2=DI, 3=MI, 4=DO, 5=FR, 6=SA, 7=SO
	volatile uint8_t Uhr_gestellt;

	
	// Fotowiderstand
	volatile uint8_t Messung_ADC;		// Schrittmerker fr ADC Statmachine
	volatile uint16_t Messung_Foto[100];	// Ringspeicher fr Messergebnisse
	volatile uint8_t Pos_Foto;		// Ringspeicherposition
	volatile uint16_t Durchschnitt_Foto;	// Durchschnittswert
	
	// 1-wire Temperatursensor
	volatile uint8_t Temp_DS_VK;		// Temperatur Vorkomma
	volatile uint8_t Temp_DS_NK;		// Temperatur Nachkomma


	// Statusvariablen
	
	volatile uint8_t Status_RK[9];		// Reedkontakte: 1=geschlossen, 0=offen
						// 0=WZ-O, 1=WZ-S, 2=K, 3=SP, 4=K1, 5=K2, 6=SZ, 7=K3, 8=DB

	volatile uint8_t Status_CAN11_DA[3];	// digitale Ausgnge
						// 0 = WZ-SD-IT West (DA6)
						// 1 = WZ-SD-Media (DA7)
						// 2 = WZ-SD-TV (DA8)

	volatile uint8_t Status_CAN12_DA[5];	// digitale Ausgnge
						// 0 = WZ-SD-Sofa Ost (DA1)
						// 1 = DL-SD-IT Nord (DA3)
						// 2 = DL-SD-Ost (DA4)
						// 3 = BK-LI (DA5)
						// 4 = BK-SD-Ost (DA6)

	volatile uint8_t Status_CAN13_DA[3];	// digitale Ausgnge
						// 0 = K-LI-Kommode (DA1)
						// 1 = K-LI-AP Ost (DA2)
						// 2 = SP-LI (DA6)

	volatile uint8_t Status_CAN14_DA[2];	// digitale Ausgnge
						// 0 = K1-SD-Tr (DA4)
						// 1 = K2-SD-Tr (DA8)

	volatile uint8_t Status_CAN15_DA[3];	// digitale Ausgnge
						// 0 = BD-LI-Spiegel (DA6)
						// 1 = WC-LI (DA7)
						// 2 = WC-MO (DA8)

	volatile uint8_t Status_CAN16_DA[5];	// digitale Ausgnge
						// 0 = K3-LI-Bad (DA2)
						// 1 = DG-LI (DA5)
						// 2 = SH-LI-EG (DA6)
						// 3 = SH-LI-OG (DA7)
						// 4 = SH-LI-Keller (DA8)

	volatile uint8_t Status_CAN17_DA[6];	// digitale Ausgnge
						// 0 = AU-LI-Tr (DA1)
						// 1 = DB-LI (DA2)
						// 2 = SS-SD-1 (DA3)
						// 3 = SS-SD-1 (DA4)
						// 4 = 24V Led Strips BD, SZ, GA, SH, WZ (DA6)
						// 5 = 24V Led Strips K, DL (DA7)
	
	volatile uint8_t Status_Dimmer[7];	// Dimmerwerte
						// 0=WZ, 1=DL, 2=K1, 3=K2, 4=SZ, 5=SZ Bett, 6=K3

	volatile uint8_t Bewegung_PM[9];	// letzte Bewegung PM
						// 0=WZ, 1=DL, 2=K, 3=GA, 4=BD, 5=WC, 6=SH, 7=DG, 8=AU
	
	volatile uint8_t Status_DL_LM[5];		// Status Led-Modul DL-LM
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

	volatile uint8_t Status_KU_LM_W[5];	// Status Led-Modul K-LM-West
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

	volatile uint8_t Status_KU_LM_S[5];	// Status Led-Modul K-LM-Sd
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

	volatile uint8_t Status_DL_LM[5];	// Status Led-Modul DL
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

	volatile uint8_t Status_GA_LM[5];	// Status Led-Modul GA-LM
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

	volatile uint8_t Status_BD_LM[5];	// Status Led-Modul BD-LM
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4
	
	volatile uint8_t Status_SH_LM[5];	// Status Led-Modul SH-LM
						// 0 = Betriebsart
						// 1 = Info 1
						// 2 = Info 2
						// 3 = Info 3
						// 4 = Info 4

#endif //_CONFIG_H


