//-------------------------------------------------------------------------------------------------------
//
// IH_EGW_v2  -  Main
//
// Einige Programmteile wurden von anderen Projekten bernommen:
//  - I2C: Peter Fleury
//  - MCP2515/CAN: Fabian Greif (www.kreatives-chaos.de)
//  - Stack und ENC28J60: Ulrich Radig
//
// Funktionen:
//	- Sendet im Sekundentakt die aktuelle Uhrzeit und das aktuelle Datum.
//	- Die Sommer- und Winterzeit wird automatisch umgestellt.
//	- Bei Steuerbefehl Stromausfall wird die Uhrzeit im EEPROM gespeichert.
//	  Die Uhrzeit luft Batteriegepuffert weiter.
//	- Die Uhrzeit kann mit dem CAN-Telegramm Nr. 82 oder per Ethernet eingestellt werden.
//	- Alle empfangene CAN Telegramm werden ber Ethernet zum PC geschickt.
//	- CAN Telegramme knnen ber Ethernet vom PC verschickt werden.
//
//-------------------------------------------------------------------------------------------------------


#include "config.h"

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "enc28j60.h"
#include "stack.h"
#include "udp.h"
#include "ntp.h"
#include "MCP2515.h"
#include "Error.h"
#include "EEPROM.h"
#include "Uhr.h"
#include "I2Cmaster.h"
#include "One-wire.h"

#include <util/delay.h>


//-------------------------------------------------------------------------------------------------------
// Funktionsprototypen

ISR(ADC_vect);  // Analog-Digital Conversation fertig



//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  
	unsigned char reset_counter = 0;  // Merker fr Ethernet-Fehler-Zhler
	uint8_t i=0;
	
	uint8_t Pulsschlag = 0;		// Merker fr Statusled blinken
	uint8_t EEPROM_Daten[4];	// zum Beschreiben und Auslesen des EEPROMs
	uint8_t EEPROM_Laenge;		// Anzahl der Datenbytes
	uint8_t adresse_h;			// EEPROM Speicheradresse High
	uint16_t adresse_l;			// EEPROM Speicheradresse Low
	uint8_t Minute_alt = 0;
	uint8_t	NTP_Merker = 0;
	uint8_t NTP_Zaehler = 0;
	uint8_t EEPROM_Stunde;
	uint8_t EEPROM_Minute;
	uint8_t Array_durch_Foto = 0;	// Merker, ob Ringspeicher fr Fotowiderstand voll ist
	uint8_t Messmerker = 0;
	uint8_t Sendemerker = 0;
	uint8_t Senderperiode = 2;  	// 2 Minuten
	uint8_t Anzahl_Foto = 30;	// Anzahl der Messungen fr Durchschnittsberechnung
	uint8_t DS_Messung = 0;
	uint8_t DS_fertig = 0;
	uint8_t Speicher_DS[9];		// Zwischenspeicher fr DS
	
				
	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;
	
    unsigned long a;
	
	for(a=0;a<1000000;a++){asm("nop");};


	Uhr_gestellt = 0;

		
	// Fehleranzeige initialisieren
	F_Error_init();


	// Uhr initialisieren
	F_Uhr_init();
	

	//Applikationen starten
	stack_init();
	
	// UDP initialisieren
	udp_init();
	
	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	// NTP initialisieren
	ntp_init();
	
	// I2C und SPI initialisieren
	F_I2C_init();
	F_SPI_init();
	
	
	// MCP2515 initialisierung
	if(F_MCP2515_init() == 0)
	{
		F_Error(11, 0, 0, 0, 0, 0);  // Fehlertext: CAN Receiver initialisieren nicht mglich
	}

	// CAN Nachricht erzeugen
	tCAN message;				// zum Senden
	tCAN message_rec;			// zum Empfangen

	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 8;
	for(i=0; i<8; i++)			// alle Datenbytes auf 0 setzen
	{
		message.data[i] = 0;
	}
	
	
	// ADC initialisieren
  	ADCSRA |= (1<<ADEN) | (1<<ADIE) | (1<<ADPS0) | (1<<ADPS1) | (1<<ADPS2);
	
	//Globale Interrupts einschalten
	sei(); 

	
	_delay_ms(100);
	
	// Besttigung ber Ethernet schicken
	eth_buffer[UDP_DATA_START] = 69;		// E
	eth_buffer[UDP_DATA_START + 1] = 71;	// G
	eth_buffer[UDP_DATA_START + 2] = 87;	// W
	eth_buffer[UDP_DATA_START + 3] = 32;	//
	eth_buffer[UDP_DATA_START + 4] = 103;	// g
	eth_buffer[UDP_DATA_START + 5] = 101;	// e
	eth_buffer[UDP_DATA_START + 6] = 115;	// s
	eth_buffer[UDP_DATA_START + 7] = 116;	// t
	eth_buffer[UDP_DATA_START + 8] = 97;	// a
	eth_buffer[UDP_DATA_START + 9] = 114;	// r
	eth_buffer[UDP_DATA_START + 10] = 116;	// t
	eth_buffer[UDP_DATA_START + 11] = 101;	// e
	eth_buffer[UDP_DATA_START + 12] = 116;	// t
	
	create_new_udp_packet(13, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
	

	
	// Uhrzeit ber NTP anfordern
	// ntp_request();
	// SET(LED_R);

	// Uhrzeit auf bestimmten Wert stellen
	Ms10 = 0;
	Sekunde = 1;
	Minute = 0;
	Stunde = 0;
	Tag = 1;
	Monat = 1;
	Jahr = 16;
	Wochentag = 5;	// Freitag
		
	F_Uhr_stellen();

	/* NTP wird zur Zeit nicht genutzt, da eine genaue Uhrzeit noch nicht so wichtig ist
	while(Uhr_gestellt == 0)
	{
		// Ethernet abfragen
	    	eth_get_data();
		
        	if(ping.result)
        	{
           	 	ping.result = 0;
        	}
		
		// wird sekndlich ausgefhrt
		if(Ms10 == 0 && Pulsschlag == 0)
		{
			Pulsschlag = 1;
			
			// Ethernetfehler-Counter
			if(eth.no_reset)
			{
				reset_counter = 0;
				eth.no_reset = 0;
			}
			else
			{
				if((reset_counter++)>5)
				{
					reset_counter = 0;
					enc_init();
				}
			}
			
			if(Sekunde % 4 == 0)
			{
				// NTP alle 4 Sekunden anfragen wenn keine Antwort
				ntp_request();
				NTP_Zaehler++;
			}

			if(NTP_Zaehler >= 5)
			{
				Uhr_gestellt = 2;  // Schleife beenden da 5x keine Antwort vom NTP Server

				// Info ber Ethernet schicken
				eth_buffer[UDP_DATA_START] = 78;	// N
				eth_buffer[UDP_DATA_START + 1] = 84;	// T
				eth_buffer[UDP_DATA_START + 2] = 80;	// P
				eth_buffer[UDP_DATA_START + 3] = 32;	//
				eth_buffer[UDP_DATA_START + 4] = 107;	// k
				eth_buffer[UDP_DATA_START + 5] = 101;	// e
				eth_buffer[UDP_DATA_START + 6] = 105;	// i
				eth_buffer[UDP_DATA_START + 7] = 110;	// n
				eth_buffer[UDP_DATA_START + 8] = 101;	// e
				eth_buffer[UDP_DATA_START + 9] = 32;	// 
				eth_buffer[UDP_DATA_START + 10] = 65;	// A
				eth_buffer[UDP_DATA_START + 11] = 110;	// n
				eth_buffer[UDP_DATA_START + 12] = 116;	// t
				eth_buffer[UDP_DATA_START + 13] = 119;	// w
				eth_buffer[UDP_DATA_START + 14] = 111;	// o
				eth_buffer[UDP_DATA_START + 15] = 114;	// r
				eth_buffer[UDP_DATA_START + 16] = 116;	// t
	
				create_new_udp_packet(17, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
			}
		}
		
		if(Ms10 == 1)
			Pulsschlag = 0;
	}

	_delay_ms(100);*/
	
	// Zeitdauer Stromaus prfen
	if(Uhr_gestellt == 1)	// Zeit von NTP
	{
		EEPROM_Stunde = F_EEPROM_lesen(0, 0);
		EEPROM_Minute = F_EEPROM_lesen(0, 1);
		
		
	}
	
	// RESET(LED_R);
	
    // Endlosschleife
    while(1)
	{
		// Pulsschlag
		if(Ms10 == 0 && Pulsschlag == 0)
		{
			if(Error_Stand == 0)  // kein Fehler zum Anzeigen
			{
				if((Sekunde % 2) == 0)
					SET(LED_G);
				else
					RESET(LED_G);	
			}
			
			Pulsschlag = 1;
			
			// Ethernetfehler-Counter
			if(eth.no_reset)
			{
				reset_counter = 0;
				eth.no_reset = 0;
			}
			else
			{
				if((reset_counter++)>5)
				{
					reset_counter = 0;
					enc_init();
				}
			}
			
			// Uhrzeit und Datum per CAN verschicken
			message.id = CAN_Adresse;
			message.header.rtr = 0;
			message.header.length = 8;
			message.data[0] = 99;
			message.data[1] = 80;
			message.data[2] = Stunde;
			message.data[3] = Minute;
			message.data[4] = Sekunde;
			message.data[5] = Tag;
			message.data[6] = Wochentag | (Monat<<4);
			message.data[7] = Jahr;

			F_MCP2515_tel_senden(&message);
		}
		
		if(Ms10 == 1)
			Pulsschlag = 0;
		
		
		if(Minute != Minute_alt)
		{
			/*// Uhrzeit ber Ethernet schicken
			eth_buffer[UDP_DATA_START] = F_InttoChar_Zehner(Stunde);
			eth_buffer[UDP_DATA_START + 1] = F_InttoChar_Einer(Stunde);
			eth_buffer[UDP_DATA_START + 2] = 58;	// :
			eth_buffer[UDP_DATA_START + 3] = F_InttoChar_Zehner(Minute);
			eth_buffer[UDP_DATA_START + 4] = F_InttoChar_Einer(Minute);
			eth_buffer[UDP_DATA_START + 5] = 58;	// :
			eth_buffer[UDP_DATA_START + 6] = F_InttoChar_Zehner(Sekunde);
			eth_buffer[UDP_DATA_START + 7] = F_InttoChar_Einer(Sekunde);
			eth_buffer[UDP_DATA_START + 8] = 32;	// Space
			eth_buffer[UDP_DATA_START + 9] = F_InttoChar_Zehner(Tag);
			eth_buffer[UDP_DATA_START + 10] = F_InttoChar_Einer(Tag);
			eth_buffer[UDP_DATA_START + 11] = 46;	// .
			eth_buffer[UDP_DATA_START + 12] = F_InttoChar_Zehner(Monat);
			eth_buffer[UDP_DATA_START + 13] = F_InttoChar_Einer(Monat);
			eth_buffer[UDP_DATA_START + 14] = 46;	// .
			eth_buffer[UDP_DATA_START + 15] = F_InttoChar_Zehner(Jahr);
			eth_buffer[UDP_DATA_START + 16] = F_InttoChar_Einer(Jahr);
			eth_buffer[UDP_DATA_START + 17] = 32;	// Space
			
			switch(Wochentag)
			{
				case 1:	// Montag
						eth_buffer[UDP_DATA_START + 18] = 77;	// M
						eth_buffer[UDP_DATA_START + 19] = 79;	// O
						break;
						
				case 2:	// Dienstag
						eth_buffer[UDP_DATA_START + 18] = 68;	// D
						eth_buffer[UDP_DATA_START + 19] = 73;	// I
						break;
				
				case 3:	// Mittwoch
						eth_buffer[UDP_DATA_START + 18] = 77;	// M
						eth_buffer[UDP_DATA_START + 19] = 73;	// I
						break;
				
				case 4:	// Donnerstag
						eth_buffer[UDP_DATA_START + 18] = 68;	// D
						eth_buffer[UDP_DATA_START + 19] = 79;	// O
						break;
				
				case 5:	// Freitag
						eth_buffer[UDP_DATA_START + 18] = 70;	// F
						eth_buffer[UDP_DATA_START + 19] = 82;	// R
						break;
				
				case 6:	// Samastag
						eth_buffer[UDP_DATA_START + 18] = 83;	// S
						eth_buffer[UDP_DATA_START + 19] = 65;	// A
						break;
				
				case 7:	// Sonntag
						eth_buffer[UDP_DATA_START + 18] = 83;	// S
						eth_buffer[UDP_DATA_START + 19] = 79;	// O
						break;
			}
			
			create_new_udp_packet(20, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
			
			Minute_alt = Minute;*/
			
			
			// Uhrzeit im EEPROM abspeichern
			F_EEPROM_schreiben(0, 0, Stunde);
			F_EEPROM_schreiben(0, 1, Minute);
		}

		
		/*// tglich Zeit ber NTP aktualisieren
		if(Stunde == 4  && NTP_Merker == 0)
		{
			NTP_Merker = 1;
			ntp_request();
		}

		if(Stunde == 5 && NTP_Merker == 1)
			NTP_Merker = 0;
		*/

		// Fehlerausgabe
		F_Error_Ausgabe();
		
		
		// Ethernet abfragen
	    eth_get_data();
		
        if(ping.result)
        {
            ping.result = 0;
        }
		
		
		// CAN Telegramm empfangen
		if(MCP2515_tel_empfangen > 0)  // CAN Telegramm empfangen
		{
			//cli();  // Interrupts ausschalten
			
			F_MCP2515_tel_empfangen(&message_rec);
			MCP2515_tel_empfangen--;
			
			/*// jedes Telegramm per UDP zum PC schicken
			// create_new_udp_packet(1, 7602, 7600, 0x6400A8C0);  // 192.168.0.100
			
			eth_buffer[UDP_DATA_START] = 82;	// Telegrammtyp R
			eth_buffer[UDP_DATA_START+1] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+2] = F_InttoChar_Hunderter(message_rec.id);	// CAN Adresse Sender
			eth_buffer[UDP_DATA_START+3] = F_InttoChar_Zehner(message_rec.id);
			eth_buffer[UDP_DATA_START+4] = F_InttoChar_Einer(message_rec.id);
			eth_buffer[UDP_DATA_START+5] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+6] = F_InttoChar_Hunderter(message_rec.data[0]);	// CAN Byte 0
			eth_buffer[UDP_DATA_START+7] = F_InttoChar_Zehner(message_rec.data[0]);
			eth_buffer[UDP_DATA_START+8] = F_InttoChar_Einer(message_rec.data[0]);
			eth_buffer[UDP_DATA_START+9] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+10] = F_InttoChar_Hunderter(message_rec.data[1]);	// CAN Byte 1
			eth_buffer[UDP_DATA_START+11] = F_InttoChar_Zehner(message_rec.data[1]);
			eth_buffer[UDP_DATA_START+12] = F_InttoChar_Einer(message_rec.data[1]);
			eth_buffer[UDP_DATA_START+13] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+14] = F_InttoChar_Hunderter(message_rec.data[2]);	// CAN Byte 2
			eth_buffer[UDP_DATA_START+15] = F_InttoChar_Zehner(message_rec.data[2]);
			eth_buffer[UDP_DATA_START+16] = F_InttoChar_Einer(message_rec.data[2]);
			eth_buffer[UDP_DATA_START+17] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+18] = F_InttoChar_Hunderter(message_rec.data[3]);	// CAN Byte 3
			eth_buffer[UDP_DATA_START+19] = F_InttoChar_Zehner(message_rec.data[3]);
			eth_buffer[UDP_DATA_START+20] = F_InttoChar_Einer(message_rec.data[3]);
			eth_buffer[UDP_DATA_START+21] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+22] = F_InttoChar_Hunderter(message_rec.data[4]);	// CAN Byte 4
			eth_buffer[UDP_DATA_START+23] = F_InttoChar_Zehner(message_rec.data[4]);
			eth_buffer[UDP_DATA_START+24] = F_InttoChar_Einer(message_rec.data[4]);
			eth_buffer[UDP_DATA_START+25] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+26] = F_InttoChar_Hunderter(message_rec.data[5]);	// CAN Byte 5
			eth_buffer[UDP_DATA_START+27] = F_InttoChar_Zehner(message_rec.data[5]);
			eth_buffer[UDP_DATA_START+28] = F_InttoChar_Einer(message_rec.data[5]);
			eth_buffer[UDP_DATA_START+29] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+30] = F_InttoChar_Hunderter(message_rec.data[6]);	// CAN Byte 6
			eth_buffer[UDP_DATA_START+31] = F_InttoChar_Zehner(message_rec.data[6]);
			eth_buffer[UDP_DATA_START+32] = F_InttoChar_Einer(message_rec.data[6]);
			eth_buffer[UDP_DATA_START+33] = 59;	// Trennzeichen ;
			eth_buffer[UDP_DATA_START+34] = F_InttoChar_Hunderter(message_rec.data[7]);	// CAN Byte 7
			eth_buffer[UDP_DATA_START+35] = F_InttoChar_Zehner(message_rec.data[7]);
			eth_buffer[UDP_DATA_START+36] = F_InttoChar_Einer(message_rec.data[7]);
			eth_buffer[UDP_DATA_START+37] = 33;	// Endzeichen !
			
			create_new_udp_packet(38, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
			
			sei();  // Interrupts einschalten*/
			
			
			// Auswertung CAN Telegramm
			if(message_rec.data[0] == CAN_Adresse || message_rec.data[0] == 99)  // Prfen, ob die Adresse passt
			{
				switch(message_rec.data[1])  // Telegrammtyp checken
				{
					case 20:	// LM Status abspeichern
							switch(message_rec.id)
							{
								case 60:	// Status von KU-LM-West
										Status_KU_LM_W[0] = message_rec.data[2];
										Status_KU_LM_W[1] = message_rec.data[3];
										Status_KU_LM_W[2] = message_rec.data[4];
										Status_KU_LM_W[3] = message_rec.data[5];
										Status_KU_LM_W[4] = message_rec.data[6];
										break;


								case 61:	// Status von KU-LM-Sd
										Status_KU_LM_S[0] = message_rec.data[2];
										Status_KU_LM_S[1] = message_rec.data[3];
										Status_KU_LM_S[2] = message_rec.data[4];
										Status_KU_LM_S[3] = message_rec.data[5];
										Status_KU_LM_S[4] = message_rec.data[6];
										break;


								case 62:	// Status von DL-LM
										Status_DL_LM[0] = message_rec.data[2];
										Status_DL_LM[1] = message_rec.data[3];
										Status_DL_LM[2] = message_rec.data[4];
										Status_DL_LM[3] = message_rec.data[5];
										Status_DL_LM[4] = message_rec.data[6];
										break;


								case 64:	// Status von BD-LM
										Status_BD_LM[0] = message_rec.data[2];
										Status_BD_LM[1] = message_rec.data[3];
										Status_BD_LM[2] = message_rec.data[4];
										Status_BD_LM[3] = message_rec.data[5];
										Status_BD_LM[4] = message_rec.data[6];
										break;


								case 65:	// Status von GA-LM
										Status_GA_LM[0] = message_rec.data[2];
										Status_GA_LM[1] = message_rec.data[3];
										Status_GA_LM[2] = message_rec.data[4];
										Status_GA_LM[3] = message_rec.data[5];
										Status_GA_LM[4] = message_rec.data[6];
										break;


								case 67:	// Status von SH-LM
										Status_SH_LM[0] = message_rec.data[2];
										Status_SH_LM[1] = message_rec.data[3];
										Status_SH_LM[2] = message_rec.data[4];
										Status_SH_LM[3] = message_rec.data[5];
										Status_SH_LM[4] = message_rec.data[6];
										break;


								default: break;
							}
							break;					


					case 21:	// CON Status abspeichern
							switch(message_rec.id)
							{
								case 11:	// Status von CON11 - WZ
										Status_RK[0] = message_rec.data[2] & 0x01;		// WZ-RK-Ost
										Status_RK[1] = (message_rec.data[2] & 0x02) >> 1;	// WZ-RK-Sd
										Status_CAN11_DA[0] = (message_rec.data[2] & 0x04) >> 2;	// WZ-SD-IT West
										Status_CAN11_DA[1] = (message_rec.data[2] & 0x08) >> 3;	// WZ-SD-Media
										Status_CAN11_DA[2] = (message_rec.data[2] & 0x10) >> 4;	// WZ-SD-TV
										Bewegung_PM[0] = message_rec.data[3];			// WZ-PM
										Status_Dimmer[0] = message_rec.data[4];			// WZ-LI
										break;


								case 12:	// Status von CON12 - WZ, DL, BK
										Status_CAN12_DA[0] = message_rec.data[2] & 0x01;	// WZ-SD-Sofa Ost
										Status_CAN12_DA[1] = (message_rec.data[2] & 0x02) >> 1;	// DL-SD-It Nord
										Status_CAN12_DA[2] = (message_rec.data[2] & 0x04) >> 2;	// DL-SD-Ost
										Status_CAN12_DA[3] = (message_rec.data[2] & 0x08) >> 3;	// BK-LI
										Status_CAN12_DA[4] = (message_rec.data[2] & 0x10) >> 4;	// BK-SD-Ost
										Bewegung_PM[1] = message_rec.data[3];			// DL-PM
										Status_Dimmer[1] = message_rec.data[4];			// DL-LI
										break;


								case 13:	// Status von CON13 - K, SP
										Status_RK[2] = message_rec.data[2] & 0x01;		// K-RK
										Status_RK[3] = (message_rec.data[2] & 0x02) >> 1;	// SP-RK
										Status_CAN13_DA[0] = (message_rec.data[2] & 0x04) >> 2;	// K-LI-Kommode
										Status_CAN13_DA[1] = (message_rec.data[2] & 0x08) >> 3;	// K-LI-AP Ost
										Status_CAN13_DA[2] = (message_rec.data[2] & 0x10) >> 4;	// SP-LI
										Bewegung_PM[2] = message_rec.data[3];			// K-PM
										break;


								case 14:	// Status von CON14 - K1, K2, GA
										Status_RK[4] = message_rec.data[2] & 0x01;		// K1-RK
										Status_RK[5] = (message_rec.data[2] & 0x02) >> 1;	// K2-RK
										Status_CAN14_DA[0] = (message_rec.data[2] & 0x04) >> 2;	// K1-SD-Tr
										Status_CAN14_DA[1] = (message_rec.data[2] & 0x08) >> 3;	// K2-SD-Tr
										Bewegung_PM[3] = message_rec.data[3];			// GA-PM
										Status_Dimmer[2] = message_rec.data[4];			// K1-LI
										Status_Dimmer[3] = message_rec.data[5];			// K2-LI
										break;


								case 15:	// Status von CON15 - SZ, BD, WC
										Status_RK[6] = message_rec.data[2] & 0x01;		// SZ-RK
										Status_CAN15_DA[0] = (message_rec.data[2] & 0x02) >> 1;	// BD-LI-Spiegel
										Status_CAN15_DA[1] = (message_rec.data[2] & 0x04) >> 2;	// WC-LI
										Status_CAN15_DA[2] = (message_rec.data[2] & 0x08) >> 3;	// WZ-MO
										Bewegung_PM[4] = message_rec.data[3];			// BD-PM
										Bewegung_PM[5] = message_rec.data[4];			// WC-PM
										Status_Dimmer[4] = message_rec.data[5];			// SZ-LI
										Status_Dimmer[5] = message_rec.data[6];			// SZ-LI-Bett
										break;


								case 16:	// Status von CON16 - K3, SH, DG
										Status_RK[7] = message_rec.data[2] & 0x01;		// K3-RK-Ost
										Status_CAN16_DA[0] = (message_rec.data[2] & 0x02) >> 1;	// K3-LI-Bad
										Status_CAN16_DA[1] = (message_rec.data[2] & 0x04) >> 2;	// DG-LI
										Status_CAN16_DA[2] = (message_rec.data[2] & 0x08) >> 3;	// SH-LI-EG
										Status_CAN16_DA[3] = (message_rec.data[2] & 0x10) >> 4;	// SH-LI-OG
										Status_CAN16_DA[4] = (message_rec.data[2] & 0x20) >> 5;	// SH-LI-Keller
										Bewegung_PM[6] = message_rec.data[3];			// SH-PM
										Bewegung_PM[7] = message_rec.data[4];			// DG-PM
										Status_Dimmer[6] = message_rec.data[5];			// K3-LI
										break;


								case 17:	// Status von CON17 - DB, AU, SS
										Status_RK[0] = message_rec.data[2] & 0x01;		// DB-RK
										Status_CAN17_DA[0] = (message_rec.data[2] & 0x02) >> 1;	// AU-LI-Tr
										Status_CAN17_DA[1] = (message_rec.data[2] & 0x04) >> 2;	// DB-LI
										Status_CAN17_DA[2] = (message_rec.data[2] & 0x08) >> 3;	// SS-SD-1
										Status_CAN17_DA[3] = (message_rec.data[2] & 0x10) >> 4;	// SS-SD-1
										Status_CAN17_DA[4] = (message_rec.data[2] & 0x20) >> 5;	// 24V Led Strips BD, SZ,GA, SH, WZ
										Status_CAN17_DA[5] = (message_rec.data[2] & 0x40) >> 6;	// 24V Led Strips K-DL
										Bewegung_PM[8] = message_rec.data[3];			// AU-PM
										Status_Dimmer[0] = message_rec.data[4];
										break;


								default:	break;
							}
							break;


					case 30:	// EEPROM Beschreiben
								if(message_rec.data[4] >= 1 && message_rec.data[4] <= 3)
								{
									for(i=0; i < message_rec.data[4]; i++)
									{
										F_EEPROM_schreiben(message_rec.data[2], message_rec.data[3] + i, message_rec.data[5 + i]);
									}
								}
								else
								{
									// Fehlertext: Empf. Telegramm 30: falscher Wert Datenbyte 4
									F_Error(12, message_rec.id, message_rec.data[4], 0, 0, 0);
								}
								break;


					case 31:	// EEPROM Anforderung Auslesen
								adresse_h = message_rec.data[2];
								adresse_l = message_rec.data[3];
								EEPROM_Laenge = 0;

								if(message_rec.data[4] >= 1 && message_rec.data[4] <=4)
								{
									for(i=0; i < message_rec.data[4]; i++)
									{
										EEPROM_Daten[i] = F_EEPROM_lesen(adresse_h, adresse_l);

										adresse_l++;
										if(adresse_l == 256)
										{
											adresse_l = 0;
											adresse_h++;

											if(adresse_h >= 8)
											{
												// Fehlertext: Empf. Telegramm 31: EEPORM Adresse High zu hoch
												F_Error(14, message_rec.id, message_rec.data[2], message_rec.data[3], message_rec.data[4], 0);
									
												i = message_rec.data[4];
												EEPROM_Laenge = 0;
												EEPROM_Daten[0] = 0;
												EEPROM_Daten[1] = 0;
												EEPROM_Daten[2] = 0;
												EEPROM_Daten[3] = 0;
											}
										}

										EEPROM_Laenge++;
									}
						
									// Antwort erstellen
									message.header.length = EEPROM_Laenge + 4;
									message.data[0] = message_rec.id;
									message.data[1] = 32;
									message.data[2] = message_rec.data[2];
									message.data[3] = message_rec.data[3];
						
									for(i=0; i < EEPROM_Laenge; i++)
									{
										message.data[i+4] = EEPROM_Daten[i];
									}

									// Antwort senden
									F_MCP2515_tel_senden(&message);
								}
					
								else
								{
									// Fehlertext: Empf. Telegramm 31: falscher Wert Datenbyte 4
									F_Error(13, message_rec.id, message_rec.data[4], 0, 0, 0);
								}
								break;


					case 70:	// Steuerbefehle
								switch(message_rec.data[2])
								{
									case 1:	// Softwarereset
											cli();
											wdt_enable(WDTO_15MS);
											while(1);
											break;
						
									case 2:	// Strom Aus
											
											break;
						
									case 3: // alle Fehler zurck setzen
											F_Error_init();
						
									default: break;
								}
								break;
					
					
					case 81:	// Zeit und Datum stellen
								EIMSK &= ~(1<<INT1);	// Uhr Int. ausschalten
								TIMSK1 &= ~(1<<TOIE1);	// 10ms Timer stoppen
					
								Ms10 = 0;
								Sekunde = message_rec.data[4];
								Minute = message_rec.data[3];
								Stunde = message_rec.data[2];
								Tag = message_rec.data[5];
								Monat = (message_rec.data[6] & 0xF0) >> 4;
								Jahr = message_rec.data[7];
								Wochentag = message_rec.data[6] & 0x0F;
					
								EIMSK |= (1<<INT1);		// Uhr Int. einschalten
								TCNT1 = 40536;			// Timer zurck setzen
								TIFR1 = (1 << TOV1);	// Overflow-flag lschen
								TIMSK1 |= (1<<TOIE1);	// 10ms Timer starten
								break;
					
					
					case 90:	// Fehlertelegramm
								break;
					

					default:	
								break;
				}
			}
		}


		// Messintervall 15 Sekunden
    		if(Sekunde == Messmerker)
    		{
      			Messmerker += 15;

      			if(Messmerker >= 60)
        			Messmerker -= 60;

      			Messung_ADC = 1;  // Statemachine fr ADC starten
			

			// DS18S20 Messung starten
      			if(F_OW_BusReset() == 0)  // alles OK
      			{
        			F_OW_BusWriteByte(0xCC);  // Skip ROM
        			F_OW_BusWriteByte(0x44);  // Messung starten

        			DS_Messung = 1;

        			DS_fertig = Sekunde + 2;
        			if(DS_fertig >= 60)
          				DS_fertig -= 60;
      			}
		}

		
		// Fotowiderstand auswerten    
		switch(Messung_ADC)
    		{
      			case 1: // Messung ADC4 starten - Fotowiderstand
              			ADMUX &= ~((1<<MUX0) | (1<<MUX1) | (1<<MUX3) | (1<<MUX4));
				ADMUX |= (1<<MUX2);
              			ADCSRA |= (1<<ADSC);
              			break;
      
      			case 2: // Messung ADC4 auslesen
              			Messung_Foto[Pos_Foto] = ADC;  // Messergebnis speichern

              			Pos_Foto++;  // Position im Array erhhen 
              			if(Pos_Foto >= 100)  // Position im Array zurck setzen
              			{
               				Pos_Foto = 0;
                			Array_durch_Foto = 1;
              			}

              			Messung_ADC = 0;
              			break;
      
      			default: break;
    		}

	

		// DS auswerten
		if(DS_fertig == Sekunde && DS_Messung == 1)
    		{
      			DS_Messung = 0;

      			// DS18S20 auslesen
      			if(F_OW_BusReset() == 0)  // alles OK
      			{
        			F_OW_BusWriteByte(0xCC);  // Skip ROM
        			F_OW_BusWriteByte(0xBE);  // Read Scratch Pad
    
        			for(i=0; i<9; i++)
        			{
         				Speicher_DS[i] = F_OW_BusReadByte();
        			}
      			}

      			// Berechnung der Temperatur
      			if(Speicher_DS[1] == 0)  // Temp >= 0C
      			{
        			Temp_DS_VK = Speicher_DS[0] / 2;  // Vorkomma
        			Temp_DS_NK = (Speicher_DS[0] % 2) * 5;  // Nachkomma
      			}
      			else  // Temp < 0C
      			{
        			Speicher_DS[0] = ~Speicher_DS[0];
        			Speicher_DS[0] += 1;
    
        			Temp_DS_VK = Speicher_DS[0] / 2;  // Vorkomma
        			Temp_DS_NK = (Speicher_DS[0] % 2) * 5 + 100;  // Nachkomma
      			}
		}

		
		
		// gleitenden Durchschnitt berechnen und verschicken
    		if(Minute == Sendemerker)
    		{
     			Sendemerker += Sendeperiode;  // Sendemerker neu setzen
      			if(Sendemerker >= 60)
        			Sendemerker -= 60;
		
			// Fotowiderstand

      			Summe_Foto = 0;

      			// genug Messdaten im Ringpuffer
      			if((Array_durch_Foto == 0 && (Sendeperiode * 4) < Pos_Foto && Pos_Foto > Anzahl_Foto) || Array_durch_Foto == 1)
      			{
        			// Durchschnitt berechnen
        			for(i=0; i<Anzahl_Foto; i++)
        			{
          				if(Pos_Foto >= Anzahl_Foto)
            					k = Pos_Foto - 1 - i;
          				else
          				{
            					if((Pos_Foto - 1) > i)
              						k = Pos_Foto - 1 - i;
            					else
             						k = 98 + Pos_Foto - i;
          				}

          				Summe_Foto += Messung_Foto[k];  // Bildung der Summe
        			}

        			Durchschnitt_Foto = Summe_Foto / Anzahl_Foto;  // Durchschnittsberechnung
      			}

      			else  // weniger Messdaten als im Parameter gefordert vorhanden
      			{
        			// Durchschnitt berechnen
        			for(i=0; i<Pos_Foto; i++)
        			{
          				Summe_Foto += Messung_Foto[Pos_Foto - 1 - i];  // Bildung der Summe
        			}

        			Durchschnitt_Foto = Summe_Foto / Pos_Foto;  // Durchschnittsberechnung
      			}


			message.header.length = 8;
			message.data[0] = 99;  // CAN Zieladresse
      			message.data[1] = 40;   // Telegrammtyp
      			message.data[2] = 1;    // Temperatur
      			message.data[3] = Temp_DS_VK;
      			message.data[4] = Temp_DS_NK;
      			message.data[5] = 3;	// Helligkeit
      			message.data[6] = Durchschnitt_Foto;
      			message.data[7] = Durchschnitt_Foto >> 8;

      			while(!F_MCP2515_freier_puffer())
      			{
        			// auf freien Puffer warten
      			}               
      			F_MCP2515_tel_senden(&message);
			
		}

    }//while (1)
		
return(0);
}



//-------------------------------------------------------------------------------------------------------
// Analog-Digital Conversation fertig

ISR(ADC_vect)
{
  // in den nchsten Schritt springen
  if(Messung_ADC == 1)
    Messung_ADC = 2;
}

