/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        12.11.2007
 Description:    NTP Client

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/


#include "config.h"

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "stack.h"
#include "ntp.h"


//----------------------------------------------------------------------------
//
/*PROGMEM const char  NTP_Request[] = {	0xd9,0x00,0x0a,0xfa,0x00,0x00,0x00,0x00,
									0x00,0x01,0x04,0x00,0x00,0x00,0x00,0x00,
									0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
									0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
									0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
									0xc7,0xd6,0xac,0x72,0x08,0x00,0x00,0x00,
									'%','E','N','D' };*/


PROGMEM const char  NTP_Request[] = {	0x1b,0x0e,0x01,0x00,0x00,0x00,0x00,0x00,
										0x00,0x00,0x00,0x00,0x4c,0x4f,0x43,0x4c,
										0xdb,0x73,0x0a,0x4b,0x0b,0x02,0x0c,0x3d,
										0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
										0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
										0xdb,0x73,0x0a,0x4b,0x0b,0x02,0x0c,0x3d,
										'%','E','N','D' };

//----------------------------------------------------------------------------
/**
 *	\ingroup time
 *	Initialisierung des NTP Ports (fr Daten empfang)
 *
 */
void ntp_init (void)
{
	//Port in Anwendungstabelle eintragen fr eingehende NTP Daten!
	add_udp_app (NTP_CLIENT_PORT, (void(*)(unsigned char))ntp_get);
	
	ntp_request();

	return;
}

//----------------------------------------------------------------------------
/**
 *	\ingroup time
 *	Anforderung der aktuellen Zeitinformationen von einem NTP Server
 *
 */
void ntp_request(void)
{
	unsigned int byte_count;
	PGM_P ntp_data_pointer = NTP_Request;

	for (byte_count = 0;byte_count<(MTU_SIZE-(UDP_DATA_START));byte_count++)
	{
		unsigned char b;
		b = pgm_read_byte(ntp_data_pointer++);
		eth_buffer[UDP_DATA_START + byte_count] = b;
		
		//wurde das Ende des Packetes erreicht?
		//Schleife wird abgebrochen keine Daten mehr!!
		if (strncasecmp_P("%END",ntp_data_pointer,4)==0)
		{	
			byte_count++;
			break;
		}
	}
		
	create_new_udp_packet(byte_count,123,123,0x8C00000A);	// 10.0.0.140
}



void TM_SetDayOfWeek(void)
{
	uint16_t day_of_year;
	uint16_t tmp_dow;

	// Day of year
	day_of_year = 31 * (Monat-1) + Tag;
	
	// Monate kleiner 31 Tage abziehen
	if (Monat > 2) {
		if ( Jahr % 4 ){	// bis zum nchsten Jahrhundertwechsel reicht das!
			day_of_year -= 3;
			} else {
			day_of_year -= 2;
		}
	}
	if (Monat >  4) {day_of_year--;}       // april
	if (Monat >  6) {day_of_year--;}       // juni
	if (Monat >  9) {day_of_year--;}       // september
	if (Monat > 11) {day_of_year--;}       // november

	// calc weekday
	tmp_dow = Jahr + ((Jahr-1) / 4) - ((Jahr-1) / 100) + day_of_year;
	if (Jahr > 0) {
		tmp_dow++;                          // 2000 war Schaltjahr
	}

	// set DOW
	Wochentag = (uint8_t) ((tmp_dow + 5) % 7);	// 31.12.1999 war Freitag (+5)
}


uint8_t const Monatstage[] PROGMEM = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	
	
uint8_t AnzahlTageImMonat(uint8_t monat, uint8_t jahr)
{
	if (monat != 2) {
		return pgm_read_byte(&Monatstage[monat-1]);
	}
	else {
		if ((jahr % 4) != 0) {	// bis zum nchsten Jahrhundertwechsel reicht das!
			return 28;
		}
		else {
			return 29;
		}
	}
}


void TM_SetDayofYear(uint16_t tage)
{
	uint8_t i;

	Monat = 1;				// Monate einzeln hochzhlen und #Tage im Monat abziehen
	i = AnzahlTageImMonat(Monat, Jahr);
	while ( tage > i ) {
		tage -= i;
		Monat += 1;
		i = AnzahlTageImMonat(Monat, Jahr);
	}
	Tag = tage;
    TM_SetDayOfWeek();
}

/*
 *   
 */

void TM_AddOneDay(void)
{
	// How many days has actual month
	uint8_t dom = AnzahlTageImMonat(Monat, Jahr);
	if (++Tag == (dom+1)) {                   // Next Month
		Tag = 1;
		if (++Monat == 13) {                    // Next year
			Monat = 1;
			Jahr++;
		}
	}

	// next day of week
	Wochentag++;
	Wochentag %= 7;
}



//----------------------------------------------------------------------------
/**
 *	\ingroup time
 *	Empfang der Zeitinformationen von einem NTP Server<br>
 *	korrigiert die empfangene Zeit auf Sommerzeit
 *
 *	\param[in]	index Zeiger auf den verwendeten Eintrag in der UDP-Tabelle
 *
 */
void ntp_get (unsigned char index)
{
	unsigned long time;	// aktuelle Zeit in Sekunden seit 1.1.1900
	
	struct NTP_GET_Header *ntp;
	ntp = (struct NTP_GET_Header *)&eth_buffer[UDP_DATA_START];

	ntp->rx_timestamp = HTONL(ntp->rx_timestamp);
	ntp->rx_timestamp += GMT_TIME_CORRECTION; //  UTC +1h
	time = ntp->rx_timestamp;
	
	uint16_t tage = (unsigned long)(time/86400);	// Tage seit 1.1.1900
	tage -= 39445;	// 108 Jahre abziehen -> Tag 0 ist 31.12.2007

	unsigned char yy = 8;		// 2008 ist Offset
	while (tage > 365) {
		tage -= 365;
		if ( yy % 4 == 0) {
			tage--;		// Schaltjahr
		}
		++yy;
	}
	Jahr = yy;

	TM_SetDayofYear(tage);		// die restlichen Tage werden direkt gesetzt.
	
	Stunde = (time/3600)%24;
	Minute = (time/60)%60;
	Sekunde = time %60;

	// auf Sommerzeit korrigieren
	if(Monat > 2 && Monat < 11) {	// 11, 12, 1 und 2 haben keine Sommerzeit

		uint8_t sommerzeit = 1;		// restliche Monate haben prinzipiell Sommerzeit

		if( (Tag - Wochentag >= 25) && (Wochentag || Stunde >= 2) ) {
			// nach 02h00 (UTC+01) des letzten Sonntags im Monat
			if( Monat == 10) {	// Oktober
				sommerzeit = 0;	// bereits Winter
			}
		}
		else {
			// vor 02h00 (UTC+01) des letzten Sonntags im Monat
			if( Monat == 3) {	// Mrz
				sommerzeit = 0;	// noch Winter
			}
		}

		Stunde += sommerzeit;	// Offset addieren

		// falls whrend der Sommerzeit zwischen 23h00 und 24h00 die Zeit neu gestellt wurde ...
		if (Stunde > 23) {
			Stunde -= 24;
			TM_AddOneDay();
		}
	}
	
	
	/*unsigned long time;	// aktuelle Zeit in Sekunden seit 1.1.1900
	uint16_t NTP_Tage;
	uint8_t NTP_Monat = 1;	
	uint8_t NTP_Jahr = 16;	// 2016 ist Offset
	uint8_t i;
	uint8_t NTP_Wochentag;
	uint8_t NTP_Stunde;
	uint8_t NTP_Minute;
	uint8_t NTP_Sekunde;
	static uint8_t Sommerzeit;


	struct NTP_GET_Header *ntp; 
	ntp = (struct NTP_GET_Header *)&eth_buffer[UDP_DATA_START];

	ntp->rx_timestamp = HTONL(ntp->rx_timestamp);
	ntp->rx_timestamp += GMT_TIME_CORRECTION; //  UTC +1h
	time = ntp->rx_timestamp;
	
	NTP_Tage = (unsigned long)(time/86400);	// Tage seit 1.1.1900
	NTP_Tage -= 42368;			// 116 Jahre abziehen -> Tag 0 ist 31.12.2015, Donnerstag


	// Wochentag berechnen
	if(NTP_Tage > 7)
		NTP_Wochentag = NTP_Tage % 7 + 4;
	
	else
		NTP_Wochentag = NTP_Tage + 4;

	if(NTP_Wochentag > 7)
		NTP_Wochentag -= 7;


	// Jahreszahl berechnen
	while(NTP_Tage > 365) 
	{
		NTP_Tage -= 365;
		if ( NTP_Jahr % 4 == 0) {
			NTP_Tage--;		// Schaltjahr
		}
		++NTP_Jahr;
	}


	// Tag und Monat berechnen
	// Monate einzeln hochzhlen und #Tage im Monat abziehen
	i = 31; // Jnner
	while(NTP_Tage > i) 
	{
		NTP_Tage -= i;
		NTP_Monat += 1;
		
		if(NTP_Monat == 3 || NTP_Monat == 5 || NTP_Monat == 7 || NTP_Monat == 8 || NTP_Monat == 10 || NTP_Monat == 12)
			i = 31;
		
		else if(NTP_Monat == 4 || NTP_Monat == 6 || NTP_Monat == 9 || NTP_Monat == 11)
			i = 30;
		
		else if(NTP_Monat == 2)
		{
			if((NTP_Jahr % 4) == 0)
				i = 29;
			
			else
				i = 28;
		}
	}

	
	NTP_Stunde = (time/3600)%24;
	NTP_Minute = (time/60)%60;
	NTP_Sekunde = time %60;


	// Sommer Umstellung
	if(NTP_Monat == 4 || NTP_Monat == 5 || NTP_Monat == 6 || NTP_Monat == 7 || NTP_Monat == 8 || NTP_Monat == 9)
		Sommerzeit = 1;

	if(NTP_Monat == 3 && NTP_Tage >= 25 && NTP_Wochentag == 7 && NTP_Stunde >= 2)	// W->S: letzter SO im Mrz
		Sommerzeit = 1;

	if(NTP_Monat == 10 && NTP_Wochentag == 7 && NTP_Tage >= 25 && NTP_Stunde >= 2) 	// S->W: letzter SO im Oktober
		Sommerzeit = 0;
	
	if(NTP_Monat == 1 || NTP_Monat == 2 || NTP_Monat == 11 || NTP_Monat == 12)
		Sommerzeit = 0;

	if(Sommerzeit == 1)
	{
		NTP_Stunde++;
		if(NTP_Stunde > 23)
		{
			NTP_Tage++;
			// 31 Tage
				if(NTP_Monat == 1 || NTP_Monat == 3 || NTP_Monat == 5 || NTP_Monat == 7 || NTP_Monat == 8 || NTP_Monat == 10 || NTP_Monat == 12)
				{
					if(NTP_Tage > 31)
					{
						NTP_Tage = 1;
						NTP_Monat++;
					}
				}
				
				// 30 Tage
				else if(NTP_Monat == 4 || NTP_Monat == 6 || NTP_Monat == 9 || NTP_Monat == 11)
				{
					if(NTP_Tage > 30)
					{
						NTP_Tage = 1;
						NTP_Monat++;
					}
				}
				
				// 28 oder 29 Tage
				else if(NTP_Monat == 2)
				{
					if(NTP_Jahr%4 == 0)  // Schaltjahr
					{
						if(NTP_Tage > 29)
						{
							NTP_Tage = 1;
							NTP_Monat++;
						}	
					}
					
					else
					{
						if(NTP_Tage > 28)
						{
							NTP_Tage = 1;
							NTP_Monat++;
						}
					}
				}
				
				
				NTP_Wochentag++;
				if(NTP_Wochentag > 7)
					NTP_Wochentag = 1;
		}
	}
	

	Sekunde = NTP_Sekunde;
	Minute = NTP_Minute;
	Stunde = NTP_Stunde;
	Jahr = NTP_Jahr;
	Monat = NTP_Monat;
	Tag = NTP_Tage;
	Wochentag = NTP_Wochentag;

	Ms10 = 0;
	TCNT1 = 40536;*/

	Uhr_gestellt = 1;
}


