/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        02.12.2007
 Description:    empfangene UDP Daten auf Port 7600 werden weiter verarbeitet

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "config.h"
#include "udp.h"
#include "stack.h"
#include "Error.h"
#include "MCP2515.h"
	
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>


//-------------------------------------------------------------------------------------------------------
// Funktionsprototypen
uint8_t F_ChartoInt (unsigned char Hunderter, unsigned char Zehner, unsigned char Einer);  // Char->Int
uint8_t F_InttoChar_Hunderter (uint8_t Wert);	// Konvertierung Integer in Char-Zeichen (Hunderterstelle)
uint8_t F_InttoChar_Zehner (uint8_t Wert);		// Konvertierung Integer in Char-Zeichen (Zehnerstelle)
uint8_t F_InttoChar_Einer (uint8_t Wert);		// Konvertierung Integer in Char-Zeichen (Einerstelle)


//----------------------------------------------------------------------------
//Initialisierung des NTP Ports (fr Daten empfang)
void udp_init (void)
{
	//Port in Anwendungstabelle eintragen fr eingehende NTP Daten!
	add_udp_app (UDP_PORT, (void(*)(unsigned char))udp_get);
	return;
}

//----------------------------------------------------------------------------
//Empfang eines UDP Telegramms am Port 7600
void udp_get (unsigned char index)
{
	uint8_t i=0;
	
	// Byte 1: Ethernettelegrammtyp
	// Byte 2-n: Daten
	
	// CAN Nachricht erzeugen
	tCAN message;				// zum Senden

	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 8;
	
	
	switch(eth_buffer[UDP_DATA_START])
	{
		case 84:	// Telegrammtyp T: CAN Nachricht senden
					// Byte 2-4: CAN Byte 1
					// Byte 6-8: CAN Byte 2
					// ...
		
					// CAN Telegramm fllen
					for(i=0; i<8; i++)
					{
						message.data[i] = F_ChartoInt(eth_buffer[UDP_DATA_START+2+i*4], eth_buffer[UDP_DATA_START+3+i*4], eth_buffer[UDP_DATA_START+4+i*4]);
					}
		
					// CAN senden
					F_MCP2515_tel_senden(&message);
					
					// Besttigung ber Ethernet schicken
					eth_buffer[UDP_DATA_START] = 67;		// C
					eth_buffer[UDP_DATA_START + 1] = 65;	// A
					eth_buffer[UDP_DATA_START + 2] = 78;	// N
					eth_buffer[UDP_DATA_START + 3] = 32;	//
					eth_buffer[UDP_DATA_START + 4] = 115;	// s
					eth_buffer[UDP_DATA_START + 5] = 101;	// e
					eth_buffer[UDP_DATA_START + 6] = 110;	// n
					eth_buffer[UDP_DATA_START + 7] = 116;	// t
					
					create_new_udp_packet(8, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
					break;
		
		
		case 85:	// Telegrammtyp U: Uhrzeit stellen
					EIMSK &= ~(1<<INT1);	// Uhr Int. ausschalten
					TIMSK1 &= ~(1<<TOIE1);	// 10ms Timer stoppen
		
					Ms10 = 0;
					Stunde = F_ChartoInt(0, eth_buffer[UDP_DATA_START+2], eth_buffer[UDP_DATA_START+3]);
					Minute = F_ChartoInt(0, eth_buffer[UDP_DATA_START+5], eth_buffer[UDP_DATA_START+6]);
					Sekunde = F_ChartoInt(0, eth_buffer[UDP_DATA_START+8], eth_buffer[UDP_DATA_START+9]);
					Tag = F_ChartoInt(0, eth_buffer[UDP_DATA_START+11], eth_buffer[UDP_DATA_START+12]);
					Monat = F_ChartoInt(0, eth_buffer[UDP_DATA_START+14], eth_buffer[UDP_DATA_START+15]);
					Jahr = F_ChartoInt(0, eth_buffer[UDP_DATA_START+17], eth_buffer[UDP_DATA_START+18]);
					Wochentag = F_ChartoInt(0, 0, eth_buffer[UDP_DATA_START+20]);
		
					EIMSK |= (1<<INT1);		// Uhr Int. einschalten
					TCNT1 = 40536;			// Timer zurck setzen
					TIFR1 = (1 << TOV1);	// Overflow-flag lschen
					TIMSK1 |= (1<<TOIE1);	// 10ms Timer starten
					
					// Besttigung ber Ethernet schicken
					eth_buffer[UDP_DATA_START] = 85;		// U
					eth_buffer[UDP_DATA_START + 1] = 104;	// h
					eth_buffer[UDP_DATA_START + 2] = 114;	// r
					eth_buffer[UDP_DATA_START + 3] = 32;	//
					eth_buffer[UDP_DATA_START + 4] = 103;	// g
					eth_buffer[UDP_DATA_START + 5] = 101;	// e
					eth_buffer[UDP_DATA_START + 6] = 115;	// s
					eth_buffer[UDP_DATA_START + 7] = 116;	// t
					eth_buffer[UDP_DATA_START + 8] = 101;	// e
					eth_buffer[UDP_DATA_START + 9] = 108;	// l
					eth_buffer[UDP_DATA_START + 10] = 108;	// l
					eth_buffer[UDP_DATA_START + 11] = 116;	// t
					
					create_new_udp_packet(12, 7600, 7600, 0xFFFFFFFF);  // 255.255.255.255
					break;
		
		
		case 67:	// Telegrammtyp C: Steuertelegramm
					switch(eth_buffer[UDP_DATA_START+2])
					{
						case 82:	// R: Softwarereset
								cli();
								wdt_enable(WDTO_15MS);
								while(1);  // Warten auf Watchdog
								break;
						
						case 83:	// S: Anforderung Gesamtstatus
								break;

						case 69:	// E: CAN loggen ein
								break;

						case 65:	// A: CAN loggen aus
								break;

						default: break;
					}
					
					break;
		
		
		case 114:	// r, Led rot einschalten
					PORTC |= (1<<3);  // SET Led rot
					break;
					
		
		case 103:	// g, Led rot ausschalten
					PORTC &= ~(1<<3);  // SET Led rot
					break;
		
		
		default:	break;
	}
}


//----------------------------------------------------------------------------
// Konvertierung von Char zu Integer
uint8_t F_ChartoInt (unsigned char Hunderter, unsigned char Zehner, unsigned char Einer)
{
	uint8_t Ergebnis = 0;
	
	switch(Hunderter)
	{
		case 48:	Ergebnis = 0; break;	// 0
		case 49:	Ergebnis = 100; break;	// 1
		case 50:	Ergebnis = 200; break;	// 2
		default:	break;
	}
	
	switch(Zehner)
	{
		case 48:	Ergebnis += 0; break;	// 0
		case 49:	Ergebnis += 10; break;	// 1
		case 50:	Ergebnis += 20; break;	// 2
		case 51:	Ergebnis += 30; break;	// 3
		case 52:	Ergebnis += 40; break;	// 4
		case 53:	Ergebnis += 50; break;	// 5
		case 54:	Ergebnis += 60; break;	// 6
		case 55:	Ergebnis += 70; break;	// 7
		case 56:	Ergebnis += 80; break;	// 8
		case 57:	Ergebnis += 90; break;	// 9
		default:	break;
	}
	
	switch(Einer)
	{
		case 48:	Ergebnis += 0; break;	// 0
		case 49:	Ergebnis += 1; break;	// 1
		case 50:	Ergebnis += 2; break;	// 2
		case 51:	Ergebnis += 3; break;	// 3
		case 52:	Ergebnis += 4; break;	// 4
		case 53:	Ergebnis += 5; break;	// 5
		case 54:	Ergebnis += 6; break;	// 6
		case 55:	Ergebnis += 7; break;	// 7
		case 56:	Ergebnis += 8; break;	// 8
		case 57:	Ergebnis += 9; break;	// 9
		default:	break;
	}
	
	return Ergebnis;
}


//-------------------------------------------------------------------------------------------------------
// Konvertierung Integer in Char-Zeichen (Hunderterstelle)
uint8_t F_InttoChar_Hunderter (uint8_t Wert)
{
	uint8_t Ergebnis;
	
	Ergebnis = Wert/100;
	
	switch(Ergebnis)
	{
		case 0: Ergebnis = 48; break;
		case 1: Ergebnis = 49; break;
		case 2: Ergebnis = 50; break;
		default: Ergebnis = 48; break;
	}
	
	return Ergebnis;
}


//-------------------------------------------------------------------------------------------------------
// Konvertierung Integer in Char-Zeichen (Zehnerstelle)
uint8_t F_InttoChar_Zehner (uint8_t Wert)
{
	uint8_t Ergebnis_1;
	uint8_t Ergebnis;
	
	Ergebnis_1 = Wert/100;
	Ergebnis = (Wert - Ergebnis_1 * 100) / 10;
	
	switch(Ergebnis)
	{
		case 0: Ergebnis = 48; break;
		case 1: Ergebnis = 49; break;
		case 2: Ergebnis = 50; break;
		case 3: Ergebnis = 51; break;
		case 4: Ergebnis = 52; break;
		case 5: Ergebnis = 53; break;
		case 6: Ergebnis = 54; break;
		case 7: Ergebnis = 55; break;
		case 8: Ergebnis = 56; break;
		case 9: Ergebnis = 57; break;
		default: Ergebnis = 48; break;
	}
	
	return Ergebnis;
}


//-------------------------------------------------------------------------------------------------------
// Konvertierung Integer in Char-Zeichen (Einerstelle)
uint8_t F_InttoChar_Einer (uint8_t Wert)
{
	uint8_t Ergebnis_1;
	uint8_t Ergebnis_2;
	uint8_t Ergebnis;
	
	Ergebnis_1 = Wert/100;
	Ergebnis_2 = (Wert - Ergebnis_1 * 100) / 10;
	Ergebnis = Wert - Ergebnis_1 * 100 - Ergebnis_2 * 10;
	
	switch(Ergebnis)
	{
		case 0: Ergebnis = 48; break;
		case 1: Ergebnis = 49; break;
		case 2: Ergebnis = 50; break;
		case 3: Ergebnis = 51; break;
		case 4: Ergebnis = 52; break;
		case 5: Ergebnis = 53; break;
		case 6: Ergebnis = 54; break;
		case 7: Ergebnis = 55; break;
		case 8: Ergebnis = 56; break;
		case 9: Ergebnis = 57; break;
		default: Ergebnis = 48; break;
	}
	
	return Ergebnis;
}







